@@ -20,6 +20,9 @@
 #include "nsProxyRelease.h"
 #include "nsIContentPolicy.h"
 #include "nsContentUtils.h"
+#include "mozilla/dom/ContentParent.h"
+#include "mozilla/net/NeckoChild.h"
+#include "../protocol/http/nsHttpHandler.h"
 
 #include "nsIFileURL.h"
 #include "nsIURIMutator.h"
@@ -28,8 +31,8 @@
 #include "prio.h"
 #include <algorithm>
 
+#include "mozilla/Components.h"
 #include "mozilla/TaskQueue.h"
-#include "mozilla/Unused.h"
 
 using namespace mozilla;
 using namespace mozilla::net;
@@ -147,8 +150,8 @@ nsresult nsFileCopyEvent::Dispatch(nsIRunnable* callback,
   if (NS_FAILED(rv)) return rv;
 
   // Dispatch ourselves to I/O thread pool...
-  nsCOMPtr<nsIEventTarget> pool =
-      do_GetService(NS_STREAMTRANSPORTSERVICE_CONTRACTID, &rv);
+  nsCOMPtr<nsIEventTarget> pool;
+  pool = mozilla::components::StreamTransport::Service(&rv);
   if (NS_FAILED(rv)) return rv;
 
   return pool->Dispatch(this, NS_DISPATCH_NORMAL);
@@ -239,16 +242,15 @@ nsFileChannel::nsFileChannel(nsIURI* uri) : mUploadLength(0), mFileURI(uri) {}
 nsresult nsFileChannel::Init() {
   NS_ENSURE_STATE(mLoadInfo);
 
+  RefPtr<nsHttpHandler> handler = nsHttpHandler::GetInstance();
+  mChannelId = handler->NewChannelId();
+
   // If we have a link file, we should resolve its target right away.
   // This is to protect against a same origin attack where the same link file
   // can point to different resources right after the first resource is loaded.
   nsCOMPtr<nsIFile> file;
   nsCOMPtr<nsIURI> targetURI;
-#ifdef XP_WIN
-  nsAutoString fileTarget;
-#else
-  nsAutoCString fileTarget;
-#endif
+  AutoPathString fileTarget;
   nsCOMPtr<nsIFile> resolvedFile;
   bool symLink;
   nsCOMPtr<nsIFileURL> fileURL = do_QueryInterface(mFileURI);
@@ -256,13 +258,11 @@ nsresult nsFileChannel::Init() {
       NS_SUCCEEDED(file->IsSymlink(&symLink)) && symLink &&
 #ifdef XP_WIN
       NS_SUCCEEDED(file->GetTarget(fileTarget)) &&
-      NS_SUCCEEDED(
-          NS_NewLocalFile(fileTarget, true, getter_AddRefs(resolvedFile))) &&
 #else
       NS_SUCCEEDED(file->GetNativeTarget(fileTarget)) &&
-      NS_SUCCEEDED(NS_NewNativeLocalFile(fileTarget, true,
-                                         getter_AddRefs(resolvedFile))) &&
 #endif
+      NS_SUCCEEDED(NS_NewPathStringLocalFile(fileTarget,
+                                             getter_AddRefs(resolvedFile))) &&
       NS_SUCCEEDED(
           NS_NewFileURI(getter_AddRefs(targetURI), resolvedFile, nullptr))) {
     // Make an effort to match up the query strings.
@@ -270,8 +270,7 @@ nsresult nsFileChannel::Init() {
     nsCOMPtr<nsIURL> targetURL = do_QueryInterface(targetURI);
     nsAutoCString queryString;
     if (origURL && targetURL && NS_SUCCEEDED(origURL->GetQuery(queryString))) {
-      Unused
-          << NS_MutateURI(targetURI).SetQuery(queryString).Finalize(targetURI);
+      (void)NS_MutateURI(targetURI).SetQuery(queryString).Finalize(targetURI);
     }
 
     SetURI(targetURI);
@@ -406,6 +405,9 @@ nsresult nsFileChannel::OpenContentStream(bool async, nsIInputStream** result,
     }
   }
 
+  // notify "file-channel-opened" observers
+  MaybeSendFileOpenNotification();
+
   *result = nullptr;
   stream.swap(*result);
   return NS_OK;
@@ -419,8 +421,7 @@ nsresult nsFileChannel::ListenerBlockingPromise(BlockingPromise** aPromise) {
     return NS_OK;
   }
 
-  nsCOMPtr<nsIEventTarget> sts(
-      do_GetService(NS_STREAMTRANSPORTSERVICE_CONTRACTID));
+  nsCOMPtr<nsIEventTarget> sts(mozilla::components::StreamTransport::Service());
   if (!sts) {
     return FixupContentLength(true);
   }
@@ -467,7 +468,7 @@ nsresult nsFileChannel::FixupContentLength(bool async) {
 // nsFileChannel::nsISupports
 
 NS_IMPL_ISUPPORTS_INHERITED(nsFileChannel, nsBaseChannel, nsIUploadChannel,
-                            nsIFileChannel)
+                            nsIFileChannel, nsIIdentChannel, nsIChildChannel)
 
 //-----------------------------------------------------------------------------
 // nsFileChannel::nsIFileChannel
@@ -481,6 +482,63 @@ nsFileChannel::GetFile(nsIFile** file) {
   return fileURL->GetFile(file);
 }
 
+nsresult nsFileChannel::MaybeSendFileOpenNotification() {
+  // We never send the notification unless we're in a content process.
+  if (!IsNeckoChild()) {
+    return NS_OK;
+  }
+
+  // Ignore system-principal triggered file channel loads unless they're for a
+  // toplevel document load.
+  if (mLoadInfo->TriggeringPrincipal()->IsSystemPrincipal() &&
+      mLoadInfo->GetExternalContentPolicyType() !=
+          ExtContentPolicyType::TYPE_DOCUMENT) {
+    return NS_OK;
+  }
+
+  uint32_t loadFlags = 0;
+  MOZ_ALWAYS_SUCCEEDS(GetLoadFlags(&loadFlags));
+
+  LoadInfoArgs loadInfoArgs;
+  MOZ_ALWAYS_SUCCEEDS(
+      mozilla::ipc::LoadInfoToLoadInfoArgs(mLoadInfo, &loadInfoArgs));
+
+  FileChannelInfo fileChannelInfo(mURI, nsBaseChannel::OriginalURI(), loadFlags,
+                                  loadInfoArgs, mContentType, mChannelId);
+  gNeckoChild->SendNotifyFileChannelOpened(fileChannelInfo);
+  return NS_OK;
+}
+
+/* static */
+nsresult nsFileChannel::DoNotifyFileChannelOpened(
+    const nsACString& aRemoteType,
+    const mozilla::net::FileChannelInfo& aFileChannelInfo) {
+  nsCOMPtr<nsIObserverService> obsService = components::Observer::Service();
+  if (!obsService) {
+    return NS_OK;
+  }
+
+  nsCOMPtr<nsILoadInfo> loadInfo;
+  nsresult rv = mozilla::ipc::LoadInfoArgsToLoadInfo(
+      aFileChannelInfo.loadInfo(), aRemoteType, getter_AddRefs(loadInfo));
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  // Re-create a file channel in the parent process to notify
+  // file-channel-opened observers.
+  RefPtr<nsFileChannel> channel;
+  channel = new nsFileChannel(aFileChannelInfo.uri());
+  channel->SetURI(aFileChannelInfo.uri());
+  channel->SetOriginalURI(aFileChannelInfo.originalURI());
+  channel->SetLoadFlags(aFileChannelInfo.loadFlags());
+  channel->SetLoadInfo(loadInfo);
+  channel->SetContentType(aFileChannelInfo.contentType());
+  MOZ_ALWAYS_SUCCEEDS(channel->SetChannelId(aFileChannelInfo.channelId()));
+
+  obsService->NotifyObservers(static_cast<nsIIdentChannel*>(channel),
+                              "file-channel-opened", nullptr);
+  return NS_OK;
+}
+
 //-----------------------------------------------------------------------------
 // nsFileChannel::nsIUploadChannel
 
@@ -512,3 +570,36 @@ nsFileChannel::GetUploadStream(nsIInputStream** result) {
   *result = do_AddRef(mUploadStream).take();
   return NS_OK;
 }
+
+//-----------------------------------------------------------------------------
+// nsFileChannel::nsIIdentChannel
+
+NS_IMETHODIMP
+nsFileChannel::GetChannelId(uint64_t* aChannelId) {
+  *aChannelId = mChannelId;
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+nsFileChannel::SetChannelId(uint64_t aChannelId) {
+  mChannelId = aChannelId;
+  return NS_OK;
+}
+
+//-----------------------------------------------------------------------------
+// nsFileChannel::nsIChildChannel
+
+NS_IMETHODIMP
+nsFileChannel::ConnectParent(uint32_t aId) {
+  if (!IsNeckoChild()) {
+    return NS_ERROR_NOT_IMPLEMENTED;
+  }
+
+  gNeckoChild->SendConnectBaseChannel(aId);
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+nsFileChannel::CompleteRedirectSetup(nsIStreamListener* aListener) {
+  return AsyncOpen(aListener);
+}
