@@ -9,7 +9,6 @@
 #include "nsICookieService.h"
 #include "nsICookieManager.h"
 #include "nsICookie.h"
-#include <stdio.h>
 #include "plstr.h"
 #include "nsNetUtil.h"
 #include "nsIChannel.h"
@@ -20,11 +19,14 @@
 #include "nsIPrefBranch.h"
 #include "nsIPrefService.h"
 #include "mozilla/dom/Document.h"
+#include "mozilla/gtest/MozAssertions.h"
 #include "mozilla/Preferences.h"
-#include "mozilla/Unused.h"
 #include "mozilla/net/CookieJarSettings.h"
+#include "mozilla/net/CookieValidation.h"
 #include "Cookie.h"
+#include "CookieParser.h"
 #include "nsIURI.h"
+#include "nsIConsoleReportCollector.h"
 
 using namespace mozilla;
 using namespace mozilla::net;
@@ -34,42 +36,11 @@ static NS_DEFINE_CID(kPrefServiceCID, NS_PREFSERVICE_CID);
 
 // various pref strings
 static const char kCookiesPermissions[] = "network.cookie.cookieBehavior";
-static const char kPrefCookieQuotaPerHost[] = "network.cookie.quotaPerHost";
 static const char kCookiesMaxPerHost[] = "network.cookie.maxPerHost";
 
-#define OFFSET_ONE_WEEK int64_t(604800) * PR_USEC_PER_SEC
-#define OFFSET_ONE_DAY int64_t(86400) * PR_USEC_PER_SEC
-
-// Set server time or expiry time
-void SetTime(PRTime offsetTime, nsAutoCString& serverString,
-             nsAutoCString& cookieString, bool expiry) {
-  char timeStringPreset[40];
-  PRTime CurrentTime = PR_Now();
-  PRTime SetCookieTime = CurrentTime + offsetTime;
-  PRTime SetExpiryTime;
-  if (expiry) {
-    SetExpiryTime = SetCookieTime - OFFSET_ONE_DAY;
-  } else {
-    SetExpiryTime = SetCookieTime + OFFSET_ONE_DAY;
-  }
-
-  // Set server time string
-  PRExplodedTime explodedTime;
-  PR_ExplodeTime(SetCookieTime, PR_GMTParameters, &explodedTime);
-  PR_FormatTimeUSEnglish(timeStringPreset, 40, "%c GMT", &explodedTime);
-  serverString.Assign(timeStringPreset);
-
-  // Set cookie string
-  PR_ExplodeTime(SetExpiryTime, PR_GMTParameters, &explodedTime);
-  PR_FormatTimeUSEnglish(timeStringPreset, 40, "%c GMT", &explodedTime);
-  cookieString.ReplaceLiteral(
-      0, strlen("test=expiry; expires=") + strlen(timeStringPreset) + 1,
-      "test=expiry; expires=");
-  cookieString.Append(timeStringPreset);
-}
-
 void SetACookieInternal(nsICookieService* aCookieService, const char* aSpec,
-                        const char* aCookieString, bool aAllowed) {
+                        const nsTArray<const char*>& aCookieStrings,
+                        bool aAllowed) {
   nsCOMPtr<nsIURI> uri;
   NS_NewURI(getter_AddRefs(uri), aSpec);
 
@@ -77,7 +48,7 @@ void SetACookieInternal(nsICookieService* aCookieService, const char* aSpec,
   nsresult rv0;
   nsCOMPtr<nsIScriptSecurityManager> ssm =
       do_GetService(NS_SCRIPTSECURITYMANAGER_CONTRACTID, &rv0);
-  ASSERT_TRUE(NS_SUCCEEDED(rv0));
+  ASSERT_NS_SUCCEEDED(rv0);
   nsCOMPtr<nsIPrincipal> specPrincipal;
   nsCString tmpString(aSpec);
   ssm->CreateContentPrincipalFromOrigin(tmpString,
@@ -89,26 +60,40 @@ void SetACookieInternal(nsICookieService* aCookieService, const char* aSpec,
                 nsIContentPolicy::TYPE_OTHER);
 
   nsCOMPtr<nsICookieJarSettings> cookieJarSettings =
-      aAllowed ? CookieJarSettings::Create(CookieJarSettings::eRegular)
-               : CookieJarSettings::GetBlockingAll();
-  MOZ_ASSERT(cookieJarSettings);
+      aAllowed
+          ? CookieJarSettings::Create(CookieJarSettings::eRegular,
+                                      /* shouldResistFingerprinting */ false)
+          : CookieJarSettings::GetBlockingAll(
+                /* shouldResistFingerprinting */ false);
+  MOZ_RELEASE_ASSERT(cookieJarSettings);
 
   nsCOMPtr<nsILoadInfo> loadInfo = dummyChannel->LoadInfo();
   loadInfo->SetCookieJarSettings(cookieJarSettings);
 
-  nsresult rv = aCookieService->SetCookieStringFromHttp(
-      uri, nsDependentCString(aCookieString), dummyChannel);
-  EXPECT_TRUE(NS_SUCCEEDED(rv));
+  for (const char* cookieString : aCookieStrings) {
+    nsresult rv = aCookieService->SetCookieStringFromHttp(
+        uri, nsDependentCString(cookieString), dummyChannel);
+    EXPECT_NS_SUCCEEDED(rv);
+  }
 }
 
 void SetACookieJarBlocked(nsICookieService* aCookieService, const char* aSpec,
                           const char* aCookieString) {
-  SetACookieInternal(aCookieService, aSpec, aCookieString, false);
+  nsTArray<const char*> cookieStrings;
+  cookieStrings.AppendElement(aCookieString);
+  SetACookieInternal(aCookieService, aSpec, cookieStrings, false);
 }
 
 void SetACookie(nsICookieService* aCookieService, const char* aSpec,
                 const char* aCookieString) {
-  SetACookieInternal(aCookieService, aSpec, aCookieString, true);
+  nsTArray<const char*> cookieStrings;
+  cookieStrings.AppendElement(aCookieString);
+  SetACookieInternal(aCookieService, aSpec, cookieStrings, true);
+}
+
+void SetACookie(nsICookieService* aCookieService, const char* aSpec,
+                const nsTArray<const char*>& aCookieStrings) {
+  SetACookieInternal(aCookieService, aSpec, aCookieStrings, true);
 }
 
 // The cookie string is returned via aCookie.
@@ -120,12 +105,12 @@ void GetACookie(nsICookieService* aCookieService, const char* aSpec,
   nsCOMPtr<nsIIOService> service = do_GetIOService();
 
   nsCOMPtr<nsIChannel> channel;
-  Unused << service->NewChannelFromURI(
+  (void)service->NewChannelFromURI(
       uri, nullptr, nsContentUtils::GetSystemPrincipal(),
       nsContentUtils::GetSystemPrincipal(), 0, nsIContentPolicy::TYPE_DOCUMENT,
       getter_AddRefs(channel));
 
-  Unused << aCookieService->GetCookieStringFromHttp(uri, channel, aCookie);
+  (void)aCookieService->GetCookieStringFromHttp(uri, channel, aCookie);
 }
 
 // The cookie string is returned via aCookie.
@@ -136,20 +121,26 @@ void GetACookieNoHttp(nsICookieService* aCookieService, const char* aSpec,
 
   RefPtr<BasePrincipal> principal =
       BasePrincipal::CreateContentPrincipal(uri, OriginAttributes());
-  MOZ_ASSERT(principal);
+  MOZ_RELEASE_ASSERT(principal);
 
   nsCOMPtr<mozilla::dom::Document> document;
-  nsresult rv = NS_NewDOMDocument(getter_AddRefs(document),
-                                  u""_ns,   // aNamespaceURI
-                                  u""_ns,   // aQualifiedName
-                                  nullptr,  // aDoctype
-                                  uri, uri, principal,
-                                  false,    // aLoadedAsData
-                                  nullptr,  // aEventObject
-                                  DocumentFlavorHTML);
-  Unused << NS_WARN_IF(NS_FAILED(rv));
+  nsresult rv =
+      NS_NewDOMDocument(getter_AddRefs(document),
+                        u""_ns,   // aNamespaceURI
+                        u""_ns,   // aQualifiedName
+                        nullptr,  // aDoctype
+                        uri, uri, principal,
+                        mozilla::dom::LoadedAsData::No,  // aLoadedAsData
+                        nullptr,                         // aEventObject
+                        DocumentFlavor::HTML);
+  (void)NS_WARN_IF(NS_FAILED(rv));
 
-  Unused << aCookieService->GetCookieStringFromDocument(document, aCookie);
+  nsAutoString cookie;
+  ErrorResult err;
+  document->GetCookie(cookie, err);
+  EXPECT_TRUE(!err.Failed());
+
+  CopyUTF16toUTF8(cookie, aCookie);
 }
 
 // some #defines for comparison rules
@@ -190,9 +181,9 @@ void InitPrefs(nsIPrefBranch* aPrefBranch) {
   // we use the most restrictive set of prefs we can;
   // however, we don't test third party blocking here.
   aPrefBranch->SetIntPref(kCookiesPermissions, 0);  // accept all
-  // Set quotaPerHost to maxPerHost - 1, so there is only one cookie
+  // Set quotaPerHost to maxPerHost, so there is only one cookie
   // will be evicted everytime.
-  aPrefBranch->SetIntPref(kPrefCookieQuotaPerHost, 49);
+  aPrefBranch->SetIntPref(kPrefCookieQuotaPerHost, 50);
   // Set the base domain limit to 50 so we have a known value.
   aPrefBranch->SetIntPref(kCookiesMaxPerHost, 50);
 
@@ -203,6 +194,9 @@ void InitPrefs(nsIPrefBranch* aPrefBranch) {
   Preferences::SetBool("network.cookieJarSettings.unblocked_for_testing", true);
   Preferences::SetBool("dom.securecontext.allowlist_onions", false);
   Preferences::SetBool("network.cookie.sameSite.schemeful", false);
+
+  // Disable a few security checks for document.cookie
+  Preferences::SetBool("dom.cookie.testing.enabled", true);
 }
 
 TEST(TestCookie, TestCookieMain)
@@ -211,10 +205,10 @@ TEST(TestCookie, TestCookieMain)
 
   nsCOMPtr<nsICookieService> cookieService =
       do_GetService(kCookieServiceCID, &rv0);
-  ASSERT_TRUE(NS_SUCCEEDED(rv0));
+  ASSERT_NS_SUCCEEDED(rv0);
 
   nsCOMPtr<nsIPrefBranch> prefBranch = do_GetService(kPrefServiceCID, &rv0);
-  ASSERT_TRUE(NS_SUCCEEDED(rv0));
+  ASSERT_NS_SUCCEEDED(rv0);
 
   InitPrefs(prefBranch);
 
@@ -268,7 +262,7 @@ TEST(TestCookie, TestCookieMain)
   // test some variations of the domain & path, for different domains of
   // a domain cookie
   SetACookie(cookieService, "http://www.domain.com",
-             "test=domain; domain=domain.com");
+             "test=domain; domain=domain.com; sameSite=lax");
   GetACookie(cookieService, "http://domain.com", cookie);
   EXPECT_TRUE(CheckResult(cookie.get(), MUST_EQUAL, "test=domain"));
   GetACookie(cookieService, "http://domain.com.", cookie);
@@ -377,10 +371,7 @@ TEST(TestCookie, TestCookieMain)
   GetACookie(cookieService, "http://path.net/foo/", cookie);
   EXPECT_TRUE(CheckResult(cookie.get(), MUST_BE_NULL));
 
-  // bug 373228: make sure cookies with paths longer than 1024 bytes,
-  // and cookies with paths or names containing tabs, are rejected.
-  // the following cookie has a path > 1024 bytes explicitly specified in the
-  // cookie
+  // attributes with size > 1024 are ignored.
   SetACookie(
       cookieService, "http://path.net/",
       "test=path; "
@@ -419,12 +410,14 @@ TEST(TestCookie, TestCookieMain)
       "789012345678901234567890123456789012345678901234567890123456789012345678"
       "9012345678901234567890",
       cookie);
-  EXPECT_TRUE(CheckResult(cookie.get(), MUST_BE_NULL));
+  EXPECT_TRUE(CheckResult(cookie.get(), MUST_EQUAL, "test=path"));
+  GetACookie(cookieService, "http://path.net/", cookie);
+  EXPECT_TRUE(CheckResult(cookie.get(), MUST_EQUAL, "test=path"));
   // the following cookie has a path > 1024 bytes implicitly specified by the
   // uri path
   SetACookie(
       cookieService,
-      "http://path.net/"
+      "http://longpath.net/"
       "123456789012345678901234567890123456789012345678901234567890123456789012"
       "345678901234567890123456789012345678901234567890123456789012345678901234"
       "567890123456789012345678901234567890123456789012345678901234567890123456"
@@ -443,7 +436,7 @@ TEST(TestCookie, TestCookieMain)
       "test=path");
   GetACookie(
       cookieService,
-      "http://path.net/"
+      "http://longpath.net/"
       "123456789012345678901234567890123456789012345678901234567890123456789012"
       "345678901234567890123456789012345678901234567890123456789012345678901234"
       "567890123456789012345678901234567890123456789012345678901234567890123456"
@@ -462,20 +455,23 @@ TEST(TestCookie, TestCookieMain)
       cookie);
   EXPECT_TRUE(CheckResult(cookie.get(), MUST_BE_NULL));
   // the following cookie includes a tab in the path
-  SetACookie(cookieService, "http://path.net/", "test=path; path=/foo\tbar/");
-  GetACookie(cookieService, "http://path.net/foo\tbar/", cookie);
+  SetACookie(cookieService, "http://pathwithtab.net/",
+             "test=path; path=/foo\tbar/");
+  GetACookie(cookieService, "http://pathwithtab.net/foo\tbar/", cookie);
   EXPECT_TRUE(CheckResult(cookie.get(), MUST_BE_NULL));
   // the following cookie includes a tab in the name
-  SetACookie(cookieService, "http://path.net/", "test\ttabs=tab");
-  GetACookie(cookieService, "http://path.net/", cookie);
-  EXPECT_TRUE(CheckResult(cookie.get(), MUST_BE_NULL));
+  SetACookie(cookieService, "http://pathwithtab.net/", "test\ttabs=tab");
+  GetACookie(cookieService, "http://pathwithtab.net/", cookie);
+  EXPECT_TRUE(CheckResult(cookie.get(), MUST_EQUAL, "test\ttabs=tab"));
   // the following cookie includes a tab in the value - allowed
-  SetACookie(cookieService, "http://path.net/", "test=tab\ttest");
-  GetACookie(cookieService, "http://path.net/", cookie);
-  EXPECT_TRUE(CheckResult(cookie.get(), MUST_EQUAL, "test=tab\ttest"));
-  SetACookie(cookieService, "http://path.net/", "test=tab\ttest; max-age=-1");
-  GetACookie(cookieService, "http://path.net/", cookie);
-  EXPECT_TRUE(CheckResult(cookie.get(), MUST_BE_NULL));
+  SetACookie(cookieService, "http://pathwithtab.net/", "test=tab\ttest");
+  GetACookie(cookieService, "http://pathwithtab.net/", cookie);
+  EXPECT_TRUE(
+      CheckResult(cookie.get(), MUST_EQUAL, "test\ttabs=tab; test=tab\ttest"));
+  SetACookie(cookieService, "http://pathwithtab.net/",
+             "test=tab\ttest; max-age=-1");
+  GetACookie(cookieService, "http://pathwithtab.net/", cookie);
+  EXPECT_TRUE(CheckResult(cookie.get(), MUST_EQUAL, "test\ttabs=tab"));
 
   // *** expiry & deletion tests
   // XXX add server time str parsing tests here
@@ -549,9 +545,10 @@ TEST(TestCookie, TestCookieMain)
   // test the setting of multiple cookies, and test the order of precedence
   // (a later cookie overwriting an earlier one, in the same header string)
   SetACookie(cookieService, "http://multiple.cookies/",
-             "test=multiple; domain=.multiple.cookies \n test=different \n "
-             "test=same; domain=.multiple.cookies \n newtest=ciao \n "
-             "newtest=foo; max-age=-6 \n newtest=reincarnated");
+             nsTArray<const char*>{
+                 "test=multiple; domain=.multiple.cookies ", " test=different ",
+                 " test=same; domain=.multiple.cookies ", "newtest=ciao ",
+                 "newtest=foo; max-age=-6 ", " newtest=reincarnated"});
   GetACookie(cookieService, "http://multiple.cookies/", cookie);
   EXPECT_TRUE(CheckResult(cookie.get(), MUST_NOT_CONTAIN, "test=multiple"));
   EXPECT_TRUE(CheckResult(cookie.get(), MUST_CONTAIN, "test=different"));
@@ -564,7 +561,7 @@ TEST(TestCookie, TestCookieMain)
   GetACookie(cookieService, "http://multiple.cookies/", cookie);
   EXPECT_TRUE(CheckResult(cookie.get(), MUST_NOT_CONTAIN, "test=same"));
   SetACookie(cookieService, "http://multiple.cookies/",
-             "\n test=different; max-age=0 \n");
+             nsTArray<const char*>{"", " test=different; max-age=0 ", ""});
   GetACookie(cookieService, "http://multiple.cookies/", cookie);
   EXPECT_TRUE(CheckResult(cookie.get(), MUST_NOT_CONTAIN, "test=different"));
   SetACookie(cookieService, "http://multiple.cookies/",
@@ -585,13 +582,16 @@ TEST(TestCookie, TestCookieMain)
   GetACookie(cookieService, "http://parser.test/", cookie);
   EXPECT_TRUE(CheckResult(cookie.get(), MUST_BE_NULL));
   SetACookie(cookieService, "http://parser.test/",
-             "test=\"fubar! = foo;bar\\\";\" parser; domain=.parser.test; "
-             "max-age=6\nfive; max-age=2.63,");
+             nsTArray<const char*>{
+                 "test=\"fubar! = foo;bar\\\";\" parser; domain=.parser.test; "
+                 "max-age=6",
+                 "five; max-age=2.63,"});
   GetACookie(cookieService, "http://parser.test/", cookie);
   EXPECT_TRUE(CheckResult(cookie.get(), MUST_CONTAIN, R"(test="fubar! = foo)"));
   EXPECT_TRUE(CheckResult(cookie.get(), MUST_CONTAIN, "five"));
   SetACookie(cookieService, "http://parser.test/",
-             "test=kill; domain=.parser.test; max-age=0 \n five; max-age=0");
+             nsTArray<const char*>{"test=kill; domain=.parser.test; max-age=0 ",
+                                   " five; max-age=0"});
   GetACookie(cookieService, "http://parser.test/", cookie);
   EXPECT_TRUE(CheckResult(cookie.get(), MUST_BE_NULL));
 
@@ -618,8 +618,10 @@ TEST(TestCookie, TestCookieMain)
   SetACookie(cookieService, "http://multi.path.tests/",
              "test1=path; path=/one/two/three");
   SetACookie(cookieService, "http://multi.path.tests/",
-             "test2=path; path=/one \n test3=path; path=/one/two/three/four \n "
-             "test4=path; path=/one/two \n test5=path; path=/one/two/");
+             nsTArray<const char*>{"test2=path; path=/one ",
+                                   " test3=path; path=/one/two/three/four ",
+                                   " test4=path; path=/one/two ",
+                                   " test5=path; path=/one/two/"});
   SetACookie(cookieService, "http://multi.path.tests/one/two/three/four/five/",
              "test6=path");
   SetACookie(cookieService,
@@ -743,17 +745,24 @@ TEST(TestCookie, TestCookieMain)
   // *** nsICookieManager interface tests
   nsCOMPtr<nsICookieManager> cookieMgr =
       do_GetService(NS_COOKIEMANAGER_CONTRACTID, &rv0);
-  ASSERT_TRUE(NS_SUCCEEDED(rv0));
+  ASSERT_NS_SUCCEEDED(rv0);
 
   const nsCOMPtr<nsICookieManager>& cookieMgr2 = cookieMgr;
   ASSERT_TRUE(cookieMgr2);
 
+  nsCOMPtr<nsIURI> uri;
+  NS_NewURI(getter_AddRefs(uri), "https://cookie.test"_ns);
+
   mozilla::OriginAttributes attrs;
 
   // first, ensure a clean slate
-  EXPECT_TRUE(NS_SUCCEEDED(cookieMgr->RemoveAll()));
+  EXPECT_NS_SUCCEEDED(cookieMgr->RemoveAll());
+
+  nsCOMPtr<nsICookieValidation> cv;
+
   // add some cookies
-  EXPECT_TRUE(NS_SUCCEEDED(cookieMgr2->AddNative("cookiemgr.test"_ns,  // domain
+  EXPECT_TRUE(NS_SUCCEEDED(cookieMgr2->AddNative(uri,
+                                                 "cookiemgr.test"_ns,  // domain
                                                  "/foo"_ns,            // path
                                                  "test1"_ns,           // name
                                                  "yes"_ns,             // value
@@ -762,20 +771,36 @@ TEST(TestCookie, TestCookieMain)
                                                  true,       // is session
                                                  INT64_MAX,  // expiry time
                                                  &attrs,     // originAttributes
-                                                 nsICookie::SAMESITE_NONE,
-                                                 nsICookie::SCHEME_HTTPS)));
-  EXPECT_TRUE(NS_SUCCEEDED(cookieMgr2->AddNative(
-      "cookiemgr.test"_ns,             // domain
-      "/foo"_ns,                       // path
-      "test2"_ns,                      // name
-      "yes"_ns,                        // value
-      false,                           // is secure
-      true,                            // is httponly
-      true,                            // is session
-      PR_Now() / PR_USEC_PER_SEC + 2,  // expiry time
-      &attrs,                          // originAttributes
-      nsICookie::SAMESITE_NONE, nsICookie::SCHEME_HTTPS)));
-  EXPECT_TRUE(NS_SUCCEEDED(cookieMgr2->AddNative("new.domain"_ns,  // domain
+                                                 nsICookie::SAMESITE_LAX,
+                                                 nsICookie::SCHEME_HTTP,
+                                                 false,    // is partitioned
+                                                 true,     // from http
+                                                 nullptr,  // operation ID
+                                                 getter_AddRefs(cv))));
+  EXPECT_TRUE(!!cv);
+  EXPECT_EQ(CookieValidation::Cast(cv)->Result(), nsICookieValidation::eOK);
+
+  EXPECT_TRUE(NS_SUCCEEDED(
+      cookieMgr2->AddNative(uri,
+                            "cookiemgr.test"_ns,                 // domain
+                            "/foo"_ns,                           // path
+                            "test2"_ns,                          // name
+                            "yes"_ns,                            // value
+                            false,                               // is secure
+                            true,                                // is httponly
+                            true,                                // is session
+                            PR_Now() / PR_USEC_PER_MSEC + 2000,  // expiry time
+                            &attrs,  // originAttributes
+                            nsICookie::SAMESITE_LAX, nsICookie::SCHEME_HTTP,
+                            false,    // is partitioned
+                            true,     // from http
+                            nullptr,  // operation ID
+                            getter_AddRefs(cv))));
+  EXPECT_TRUE(!!cv);
+  EXPECT_EQ(CookieValidation::Cast(cv)->Result(), nsICookieValidation::eOK);
+
+  EXPECT_TRUE(NS_SUCCEEDED(cookieMgr2->AddNative(uri,
+                                                 "new.domain"_ns,  // domain
                                                  "/rabbit"_ns,     // path
                                                  "test3"_ns,       // name
                                                  "yes"_ns,         // value
@@ -784,11 +809,18 @@ TEST(TestCookie, TestCookieMain)
                                                  true,       // is session
                                                  INT64_MAX,  // expiry time
                                                  &attrs,     // originAttributes
-                                                 nsICookie::SAMESITE_NONE,
-                                                 nsICookie::SCHEME_HTTPS)));
+                                                 nsICookie::SAMESITE_LAX,
+                                                 nsICookie::SCHEME_HTTP,
+                                                 false,    // is partitioned
+                                                 true,     // from http
+                                                 nullptr,  // operation ID
+                                                 getter_AddRefs(cv))));
+  EXPECT_TRUE(!!cv);
+  EXPECT_EQ(CookieValidation::Cast(cv)->Result(), nsICookieValidation::eOK);
+
   // confirm using enumerator
   nsTArray<RefPtr<nsICookie>> cookies;
-  EXPECT_TRUE(NS_SUCCEEDED(cookieMgr->GetCookies(cookies)));
+  EXPECT_NS_SUCCEEDED(cookieMgr->GetCookies(cookies));
   nsCOMPtr<nsICookie> expiredCookie, newDomainCookie;
   for (const auto& cookie : cookies) {
     nsAutoCString name;
@@ -827,7 +859,7 @@ TEST(TestCookie, TestCookieMain)
       "cookiemgr.test"_ns, "/foo"_ns, "test2"_ns, &attrs, &found)));
   EXPECT_TRUE(found);
   // double-check RemoveAll() using the enumerator
-  EXPECT_TRUE(NS_SUCCEEDED(cookieMgr->RemoveAll()));
+  EXPECT_NS_SUCCEEDED(cookieMgr->RemoveAll());
   cookies.SetLength(0);
   EXPECT_TRUE(NS_SUCCEEDED(cookieMgr->GetCookies(cookies)) &&
               cookies.IsEmpty());
@@ -877,7 +909,7 @@ TEST(TestCookie, TestCookieMain)
 
   // *** SameSite attribute - parsing and cookie storage tests
   // Clear the cookies
-  EXPECT_TRUE(NS_SUCCEEDED(cookieMgr->RemoveAll()));
+  EXPECT_NS_SUCCEEDED(cookieMgr->RemoveAll());
 
   // None of these cookies will be set because using
   // CookieJarSettings::GetBlockingAll().
@@ -894,7 +926,7 @@ TEST(TestCookie, TestCookieMain)
                        "lax=yes; samesite=lax");
 
   cookies.SetLength(0);
-  EXPECT_TRUE(NS_SUCCEEDED(cookieMgr->GetCookies(cookies)));
+  EXPECT_NS_SUCCEEDED(cookieMgr->GetCookies(cookies));
 
   EXPECT_TRUE(cookies.IsEmpty());
 
@@ -916,7 +948,7 @@ TEST(TestCookie, TestCookieMain)
   SetACookie(cookieService, "http://samesite.test", "lax=yes; samesite=lax");
 
   cookies.SetLength(0);
-  EXPECT_TRUE(NS_SUCCEEDED(cookieMgr->GetCookies(cookies)));
+  EXPECT_NS_SUCCEEDED(cookieMgr->GetCookies(cookies));
 
   // check the cookies for the required samesite value
   for (const auto& cookie : cookies) {
@@ -925,12 +957,14 @@ TEST(TestCookie, TestCookieMain)
     int32_t sameSiteAttr;
     cookie->GetSameSite(&sameSiteAttr);
     if (name.EqualsLiteral("unset")) {
-      EXPECT_TRUE(sameSiteAttr == nsICookie::SAMESITE_NONE);
+      EXPECT_TRUE(sameSiteAttr == nsICookie::SAMESITE_UNSET);
     } else if (name.EqualsLiteral("unspecified")) {
-      EXPECT_TRUE(sameSiteAttr == nsICookie::SAMESITE_NONE);
+      EXPECT_TRUE(sameSiteAttr == nsICookie::SAMESITE_UNSET);
     } else if (name.EqualsLiteral("empty")) {
-      EXPECT_TRUE(sameSiteAttr == nsICookie::SAMESITE_NONE);
+      EXPECT_TRUE(sameSiteAttr == nsICookie::SAMESITE_UNSET);
     } else if (name.EqualsLiteral("bogus")) {
+      EXPECT_TRUE(sameSiteAttr == nsICookie::SAMESITE_UNSET);
+    } else if (name.EqualsLiteral("none")) {
       EXPECT_TRUE(sameSiteAttr == nsICookie::SAMESITE_NONE);
     } else if (name.EqualsLiteral("strict")) {
       EXPECT_TRUE(sameSiteAttr == nsICookie::SAMESITE_STRICT);
@@ -943,7 +977,7 @@ TEST(TestCookie, TestCookieMain)
 
   // *** SameSite attribute
   // Clear the cookies
-  EXPECT_TRUE(NS_SUCCEEDED(cookieMgr->RemoveAll()));
+  EXPECT_NS_SUCCEEDED(cookieMgr->RemoveAll());
 
   // please note that the flag aForeign is always set to true using this test
   // setup because no nsIChannel is passed to SetCookieString(). therefore we
@@ -981,50 +1015,43 @@ TEST(TestCookie, TestCookieMain)
   // *** speed tests
 }
 
-TEST(TestCookie, SameSiteLax)
+TEST(TestCookie, InvalidCharsInNameAndValue)
 {
-  Preferences::SetBool("network.cookie.sameSite.laxByDefault", true);
-
   nsresult rv;
 
-  nsCOMPtr<nsICookieService> cookieService =
-      do_GetService(kCookieServiceCID, &rv);
-  ASSERT_TRUE(NS_SUCCEEDED(rv));
-
   nsCOMPtr<nsICookieManager> cookieMgr =
       do_GetService(NS_COOKIEMANAGER_CONTRACTID, &rv);
-  ASSERT_TRUE(NS_SUCCEEDED(rv));
+  ASSERT_NS_SUCCEEDED(rv);
 
-  EXPECT_TRUE(NS_SUCCEEDED(cookieMgr->RemoveAll()));
+  nsCOMPtr<nsIURI> uri;
+  NS_NewURI(getter_AddRefs(uri), "https://cookie.test"_ns);
 
-  SetACookie(cookieService, "http://samesite.test", "unset=yes");
+  mozilla::OriginAttributes attrs;
 
-  nsTArray<RefPtr<nsICookie>> cookies;
-  EXPECT_TRUE(NS_SUCCEEDED(cookieMgr->GetCookies(cookies)));
-  EXPECT_EQ(cookies.Length(), (uint64_t)1);
+  // Test some invalid chars
+#define TEST_INVALID_CHARS(name, value, error)                                 \
+  {                                                                            \
+    nsCOMPtr<nsICookieValidation> cv;                                          \
+    EXPECT_EQ(                                                                 \
+        cookieMgr->AddNative(uri, "cookiemgr.test"_ns, "/foo"_ns, name, value, \
+                             false, false, true, INT64_MAX, &attrs,            \
+                             nsICookie::SAMESITE_LAX, nsICookie::SCHEME_HTTP,  \
+                             false, true, nullptr, getter_AddRefs(cv)),        \
+        NS_ERROR_ILLEGAL_VALUE);                                               \
+    EXPECT_TRUE(!!cv);                                                         \
+    EXPECT_EQ(CookieValidation::Cast(cv)->Result(), error);                    \
+  }
 
-  Cookie* cookie = static_cast<Cookie*>(cookies[0].get());
-  EXPECT_EQ(cookie->RawSameSite(), nsICookie::SAMESITE_NONE);
-  EXPECT_EQ(cookie->SameSite(), nsICookie::SAMESITE_LAX);
+  TEST_INVALID_CHARS(" test invalid name"_ns, "test valid value"_ns,
+                     nsICookieValidation::eRejectedInvalidCharName)
+  TEST_INVALID_CHARS("test invalid name "_ns, "test valid value"_ns,
+                     nsICookieValidation::eRejectedInvalidCharName)
+  TEST_INVALID_CHARS("test valid name"_ns, " test invalid value"_ns,
+                     nsICookieValidation::eRejectedInvalidCharValue)
+  TEST_INVALID_CHARS("test valid name"_ns, "test invalid value "_ns,
+                     nsICookieValidation::eRejectedInvalidCharValue)
 
-  Preferences::SetCString("network.cookie.sameSite.laxByDefault.disabledHosts",
-                          "foo.com,samesite.test,bar.net");
-
-  EXPECT_TRUE(NS_SUCCEEDED(cookieMgr->RemoveAll()));
-
-  cookies.SetLength(0);
-  EXPECT_TRUE(NS_SUCCEEDED(cookieMgr->GetCookies(cookies)));
-  EXPECT_EQ(cookies.Length(), (uint64_t)0);
-
-  SetACookie(cookieService, "http://samesite.test", "unset=yes");
-
-  cookies.SetLength(0);
-  EXPECT_TRUE(NS_SUCCEEDED(cookieMgr->GetCookies(cookies)));
-  EXPECT_EQ(cookies.Length(), (uint64_t)1);
-
-  cookie = static_cast<Cookie*>(cookies[0].get());
-  EXPECT_EQ(cookie->RawSameSite(), nsICookie::SAMESITE_NONE);
-  EXPECT_EQ(cookie->SameSite(), nsICookie::SAMESITE_LAX);
+#undef TEST_INVALID_CHARS
 }
 
 TEST(TestCookie, OnionSite)
@@ -1037,7 +1064,7 @@ TEST(TestCookie, OnionSite)
 
   nsCOMPtr<nsICookieService> cookieService =
       do_GetService(kCookieServiceCID, &rv);
-  ASSERT_TRUE(NS_SUCCEEDED(rv));
+  ASSERT_NS_SUCCEEDED(rv);
 
   // .onion secure cookie tests
   SetACookie(cookieService, "http://123456789abcdef.onion/",
@@ -1065,7 +1092,7 @@ TEST(TestCookie, HiddenPrefix)
 
   nsCOMPtr<nsICookieService> cookieService =
       do_GetService(kCookieServiceCID, &rv);
-  ASSERT_TRUE(NS_SUCCEEDED(rv));
+  ASSERT_NS_SUCCEEDED(rv);
 
   SetACookie(cookieService, "http://hiddenprefix.test/", "=__Host-test=a");
   GetACookie(cookieService, "http://hiddenprefix.test/", cookie);
@@ -1083,3 +1110,117 @@ TEST(TestCookie, HiddenPrefix)
   GetACookie(cookieService, "http://hiddenprefix.test/", cookie);
   EXPECT_TRUE(CheckResult(cookie.get(), MUST_BE_NULL));
 }
+
+TEST(TestCookie, BlockUnicode)
+{
+  Preferences::SetBool("network.cookie.blockUnicode", true);
+
+  nsresult rv;
+  nsCString cookie;
+
+  nsCOMPtr<nsICookieService> cookieService =
+      do_GetService(kCookieServiceCID, &rv);
+  ASSERT_NS_SUCCEEDED(rv);
+
+  SetACookie(cookieService, "http://unicode.com/", "name=🍪");
+  GetACookie(cookieService, "http://unicode.com/", cookie);
+  EXPECT_TRUE(CheckResult(cookie.get(), MUST_BE_NULL));
+
+  SetACookie(cookieService, "http://unicode.com/", "🍪=value");
+  GetACookie(cookieService, "http://unicode.com/", cookie);
+  EXPECT_TRUE(CheckResult(cookie.get(), MUST_BE_NULL));
+
+  Preferences::SetBool("network.cookie.blockUnicode", false);
+
+  SetACookie(cookieService, "http://unicode.com/", "name=🍪");
+  GetACookie(cookieService, "http://unicode.com/", cookie);
+  EXPECT_TRUE(CheckResult(cookie.get(), MUST_EQUAL, "name=🍪"));
+
+  nsCOMPtr<nsICookieManager> cookieMgr =
+      do_GetService(NS_COOKIEMANAGER_CONTRACTID);
+  EXPECT_NS_SUCCEEDED(cookieMgr->RemoveAll());
+
+  SetACookie(cookieService, "http://unicode.com/", "🍪=value");
+  GetACookie(cookieService, "http://unicode.com/", cookie);
+  EXPECT_TRUE(CheckResult(cookie.get(), MUST_EQUAL, "🍪=value"));
+
+  EXPECT_NS_SUCCEEDED(cookieMgr->RemoveAll());
+  Preferences::ClearUser("network.cookie.blockUnicode");
+}
+
+TEST(TestCookie, MaxAgeParser)
+{
+  nsCOMPtr<nsIURI> uri;
+  NS_NewURI(getter_AddRefs(uri), "https://maxage.net");
+
+  nsCOMPtr<nsIIOService> service = do_GetIOService();
+
+  nsCOMPtr<nsIChannel> channel;
+  (void)service->NewChannelFromURI(
+      uri, nullptr, nsContentUtils::GetSystemPrincipal(),
+      nsContentUtils::GetSystemPrincipal(), 0, nsIContentPolicy::TYPE_DOCUMENT,
+      getter_AddRefs(channel));
+
+  nsCOMPtr<nsIConsoleReportCollector> crc = do_QueryInterface(channel);
+
+  CookieParser cp(crc, uri);
+
+  int64_t value;
+  EXPECT_FALSE(cp.ParseMaxAgeAttribute(""_ns, &value));
+
+  EXPECT_TRUE(cp.ParseMaxAgeAttribute("0"_ns, &value));
+  EXPECT_EQ(value, 0);
+
+  EXPECT_TRUE(cp.ParseMaxAgeAttribute("1"_ns, &value));
+  EXPECT_EQ(value, 1);
+
+  EXPECT_TRUE(cp.ParseMaxAgeAttribute("1234"_ns, &value));
+  EXPECT_EQ(value, 1234);
+
+  EXPECT_TRUE(cp.ParseMaxAgeAttribute("00000000000000001234"_ns, &value));
+  EXPECT_EQ(value, 1234);
+
+  EXPECT_TRUE(cp.ParseMaxAgeAttribute("-1234"_ns, &value));
+  EXPECT_EQ(value, INT64_MIN);
+
+  {
+    nsCString str;
+    for (int i = 0; i < 1024; ++i) {
+      str.Append("9");
+    }
+    EXPECT_TRUE(cp.ParseMaxAgeAttribute(str, &value));
+    EXPECT_EQ(value, INT64_MAX);
+  }
+
+  EXPECT_FALSE(cp.ParseMaxAgeAttribute("1234a"_ns, &value));
+  EXPECT_FALSE(cp.ParseMaxAgeAttribute("12a34"_ns, &value));
+  EXPECT_FALSE(cp.ParseMaxAgeAttribute("12🌊34"_ns, &value));
+
+  nsresult rv;
+  nsCOMPtr<nsICookieManager> cookieMgr =
+      do_GetService(NS_COOKIEMANAGER_CONTRACTID, &rv);
+  ASSERT_NS_SUCCEEDED(rv);
+
+  EXPECT_NS_SUCCEEDED(cookieMgr->RemoveAll());
+
+  nsCOMPtr<nsICookieService> cookieService =
+      do_GetService(kCookieServiceCID, &rv);
+  ASSERT_NS_SUCCEEDED(rv);
+
+  SetACookie(cookieService, "http://maxage.net/", "a=1; max-age=1234");
+
+  nsCString cookieStr;
+  GetACookie(cookieService, "http://maxage.net/", cookieStr);
+  EXPECT_TRUE(CheckResult(cookieStr.get(), MUST_EQUAL, "a=1"));
+
+  nsTArray<RefPtr<nsICookie>> cookies;
+  EXPECT_NS_SUCCEEDED(cookieMgr->GetCookies(cookies));
+  EXPECT_EQ(cookies.Length(), (uint64_t)1);
+
+  Cookie* cookie = static_cast<Cookie*>(cookies[0].get());
+  EXPECT_FALSE(cookie->IsSession());
+
+  SetACookie(cookieService, "http://maxage.net/", "a=1; max-age=-1");
+  GetACookie(cookieService, "http://maxage.net/", cookieStr);
+  EXPECT_TRUE(CheckResult(cookieStr.get(), MUST_EQUAL, ""));
+}
