@@ -20,14 +20,14 @@ const certOverrideService = Cc[
 ].getService(Ci.nsICertOverrideService);
 
 add_setup(async function setup() {
-  h2Port = Services.env.get("MOZHTTP2_PORT");
-  Assert.notEqual(h2Port, null);
-  Assert.notEqual(h2Port, "");
-
   h3Port = Services.env.get("MOZHTTP3_PORT_NO_RESPONSE");
   Assert.notEqual(h3Port, null);
   Assert.notEqual(h3Port, "");
 
+  trrServer = new TRRServer();
+  await trrServer.start();
+  h2Port = trrServer.port();
+
   trr_test_setup();
 
   if (mozinfo.socketprocess_networking) {
@@ -38,6 +38,10 @@ add_setup(async function setup() {
   Services.prefs.setIntPref("network.trr.mode", 2); // TRR first
   Services.prefs.setBoolPref("network.http.http3.enable", true);
   Services.prefs.setIntPref("network.http.speculative-parallel-limit", 6);
+  Services.prefs.setBoolPref(
+    "network.dns.https_rr.check_record_with_cname",
+    false
+  );
 
   registerCleanupFunction(async () => {
     trr_clear_prefs();
@@ -61,6 +65,9 @@ add_setup(async function setup() {
     Services.prefs.clearUserPref(
       "network.http.http3.parallel_fallback_conn_limit"
     );
+    Services.prefs.clearUserPref(
+      "network.dns.https_rr.check_record_with_cname"
+    );
     if (trrServer) {
       await trrServer.stop();
     }
@@ -86,8 +93,6 @@ function channelOpenPromise(chan, flags, delay) {
         false
       );
     }
-    let internal = chan.QueryInterface(Ci.nsIHttpChannelInternal);
-    internal.setWaitForHTTPSSVCRecord();
     certOverrideService.setDisableAllSecurityChecksAndLetAttackersInterceptMyData(
       true
     );
@@ -102,7 +107,7 @@ function channelOpenPromise(chan, flags, delay) {
 let CheckOnlyHttp2Listener = function () {};
 
 CheckOnlyHttp2Listener.prototype = {
-  onStartRequest: function testOnStartRequest(request) {},
+  onStartRequest: function testOnStartRequest() {},
 
   onDataAvailable: function testOnDataAvailable(request, stream, off, cnt) {
     read_stream(stream, cnt);
@@ -152,7 +157,7 @@ add_task(async function test_fast_fallback_with_speculative_connection() {
   // Set AltSvc to point to not existing HTTP3 server on port 443
   Services.prefs.setCharPref(
     "network.http.http3.alt-svc-mapping-for-testing",
-    "foo.example.com;h3-29=:" + h3Port
+    "foo.example.com;h3=:" + h3Port
   );
   Services.prefs.setBoolPref("network.dns.disableIPv6", true);
   Services.prefs.setIntPref("network.http.speculative-parallel-limit", 6);
@@ -183,8 +188,6 @@ add_task(async function test_fast_fallback_without_speculative_connection() {
 // longer timeout in this test, so when fast fallback timer is triggered, the
 // http transaction is already activated.
 add_task(async function testFastfallback() {
-  trrServer = new TRRServer();
-  await trrServer.start();
   Services.prefs.setBoolPref("network.dns.upgrade_with_https_rr", true);
   Services.prefs.setBoolPref("network.dns.use_https_rr_as_altsvc", true);
   Services.prefs.setBoolPref("network.dns.echconfig.enabled", false);
@@ -192,7 +195,7 @@ add_task(async function testFastfallback() {
   Services.prefs.setIntPref("network.trr.mode", 3);
   Services.prefs.setCharPref(
     "network.trr.uri",
-    `https://foo.example.com:${trrServer.port}/dns-query`
+    `https://foo.example.com:${trrServer.port()}/dns-query`
   );
   Services.prefs.setBoolPref("network.http.http3.enable", true);
 
@@ -212,7 +215,8 @@ add_task(async function testFastfallback() {
           priority: 1,
           name: "test.fastfallback1.com",
           values: [
-            { key: "alpn", value: "h3-29" },
+            { key: "alpn", value: "h3" },
+            { key: "no-default-alpn" },
             { key: "port", value: h3Port },
             { key: "echconfig", value: "456..." },
           ],
@@ -265,15 +269,11 @@ add_task(async function testFastfallback() {
   Assert.equal(req.protocolVersion, "h2");
   let internal = req.QueryInterface(Ci.nsIHttpChannelInternal);
   Assert.equal(internal.remotePort, h2Port);
-
-  await trrServer.stop();
 });
 
 // Like the previous test, but with a shorter timeout, so when fast fallback
 // timer is triggered, the http transaction is still in pending queue.
 add_task(async function testFastfallback1() {
-  trrServer = new TRRServer();
-  await trrServer.start();
   Services.prefs.setBoolPref("network.dns.upgrade_with_https_rr", true);
   Services.prefs.setBoolPref("network.dns.use_https_rr_as_altsvc", true);
   Services.prefs.setBoolPref("network.dns.echconfig.enabled", false);
@@ -281,7 +281,7 @@ add_task(async function testFastfallback1() {
   Services.prefs.setIntPref("network.trr.mode", 3);
   Services.prefs.setCharPref(
     "network.trr.uri",
-    `https://foo.example.com:${trrServer.port}/dns-query`
+    `https://foo.example.com:${trrServer.port()}/dns-query`
   );
   Services.prefs.setBoolPref("network.http.http3.enable", true);
 
@@ -301,7 +301,8 @@ add_task(async function testFastfallback1() {
           priority: 1,
           name: "test.fastfallback1.org",
           values: [
-            { key: "alpn", value: "h3-29" },
+            { key: "alpn", value: "h3" },
+            { key: "no-default-alpn" },
             { key: "port", value: h3Port },
             { key: "echconfig", value: "456..." },
           ],
@@ -354,15 +355,12 @@ add_task(async function testFastfallback1() {
   Assert.equal(req.protocolVersion, "h2");
   let internal = req.QueryInterface(Ci.nsIHttpChannelInternal);
   Assert.equal(internal.remotePort, h2Port);
-
-  await trrServer.stop();
 });
 
 // Test when echConfig is enabled, we can sucessfully fallback to the last
 // record.
 add_task(async function testFastfallbackWithEchConfig() {
-  trrServer = new TRRServer();
-  await trrServer.start();
+  // XXX(valenting): This test takes 30s to fallback.
   Services.prefs.setBoolPref("network.dns.upgrade_with_https_rr", true);
   Services.prefs.setBoolPref("network.dns.use_https_rr_as_altsvc", true);
   Services.prefs.setBoolPref("network.dns.echconfig.enabled", true);
@@ -371,7 +369,7 @@ add_task(async function testFastfallbackWithEchConfig() {
   Services.prefs.setIntPref("network.trr.mode", 3);
   Services.prefs.setCharPref(
     "network.trr.uri",
-    `https://foo.example.com:${trrServer.port}/dns-query`
+    `https://foo.example.com:${trrServer.port()}/dns-query`
   );
   Services.prefs.setBoolPref("network.http.http3.enable", true);
 
@@ -391,7 +389,7 @@ add_task(async function testFastfallbackWithEchConfig() {
           priority: 1,
           name: "test.ech1.org",
           values: [
-            { key: "alpn", value: "h3-29" },
+            { key: "alpn", value: "h3" },
             { key: "port", value: h3Port },
             { key: "echconfig", value: "456..." },
           ],
@@ -459,15 +457,12 @@ add_task(async function testFastfallbackWithEchConfig() {
   Assert.equal(req.protocolVersion, "h2");
   let internal = req.QueryInterface(Ci.nsIHttpChannelInternal);
   Assert.equal(internal.remotePort, h2Port);
-
-  await trrServer.stop();
 });
 
 // Test when echConfig is enabled, the connection should fail when not all
 // records have echConfig.
 add_task(async function testFastfallbackWithpartialEchConfig() {
-  trrServer = new TRRServer();
-  await trrServer.start();
+  // XXX(valenting): This test takes 30s to fallback.
   Services.prefs.setBoolPref("network.dns.upgrade_with_https_rr", true);
   Services.prefs.setBoolPref("network.dns.use_https_rr_as_altsvc", true);
   Services.prefs.setBoolPref("network.dns.echconfig.enabled", true);
@@ -476,7 +471,7 @@ add_task(async function testFastfallbackWithpartialEchConfig() {
   Services.prefs.setIntPref("network.trr.mode", 3);
   Services.prefs.setCharPref(
     "network.trr.uri",
-    `https://foo.example.com:${trrServer.port}/dns-query`
+    `https://foo.example.com:${trrServer.port()}/dns-query`
   );
   Services.prefs.setBoolPref("network.http.http3.enable", true);
 
@@ -496,7 +491,7 @@ add_task(async function testFastfallbackWithpartialEchConfig() {
           priority: 1,
           name: "test.partial_ech1.org",
           values: [
-            { key: "alpn", value: "h3-29" },
+            { key: "alpn", value: "h3" },
             { key: "port", value: h3Port },
             { key: "echconfig", value: "456..." },
           ],
@@ -545,20 +540,16 @@ add_task(async function testFastfallbackWithpartialEchConfig() {
 
   let chan = makeChan(`https://test.partial_ech.org/server-timing`);
   await channelOpenPromise(chan, CL_EXPECT_LATE_FAILURE | CL_ALLOW_UNKNOWN_CL);
-
-  await trrServer.stop();
 });
 
 add_task(async function testFastfallbackWithoutEchConfig() {
-  trrServer = new TRRServer();
-  await trrServer.start();
   Services.prefs.setBoolPref("network.dns.upgrade_with_https_rr", true);
   Services.prefs.setBoolPref("network.dns.use_https_rr_as_altsvc", true);
 
   Services.prefs.setIntPref("network.trr.mode", 3);
   Services.prefs.setCharPref(
     "network.trr.uri",
-    `https://foo.example.com:${trrServer.port}/dns-query`
+    `https://foo.example.com:${trrServer.port()}/dns-query`
   );
   Services.prefs.setBoolPref("network.http.http3.enable", true);
 
@@ -578,7 +569,7 @@ add_task(async function testFastfallbackWithoutEchConfig() {
           priority: 1,
           name: "test.no_ech_h3.org",
           values: [
-            { key: "alpn", value: "h3-29" },
+            { key: "alpn", value: "h3" },
             { key: "port", value: h3Port },
           ],
         },
@@ -615,13 +606,9 @@ add_task(async function testFastfallbackWithoutEchConfig() {
   Assert.equal(req.protocolVersion, "h2");
   let internal = req.QueryInterface(Ci.nsIHttpChannelInternal);
   Assert.equal(internal.remotePort, h2Port);
-
-  await trrServer.stop();
 });
 
 add_task(async function testH3FallbackWithMultipleTransactions() {
-  trrServer = new TRRServer();
-  await trrServer.start();
   Services.prefs.setBoolPref("network.dns.upgrade_with_https_rr", true);
   Services.prefs.setBoolPref("network.dns.use_https_rr_as_altsvc", true);
   Services.prefs.setBoolPref("network.dns.echconfig.enabled", false);
@@ -629,7 +616,7 @@ add_task(async function testH3FallbackWithMultipleTransactions() {
   Services.prefs.setIntPref("network.trr.mode", 3);
   Services.prefs.setCharPref(
     "network.trr.uri",
-    `https://foo.example.com:${trrServer.port}/dns-query`
+    `https://foo.example.com:${trrServer.port()}/dns-query`
   );
   Services.prefs.setBoolPref("network.http.http3.enable", true);
 
@@ -651,7 +638,7 @@ add_task(async function testH3FallbackWithMultipleTransactions() {
           priority: 1,
           name: "test.multiple_trans.org",
           values: [
-            { key: "alpn", value: "h3-29" },
+            { key: "alpn", value: "h3" },
             { key: "port", value: h3Port },
           ],
         },
@@ -686,13 +673,9 @@ add_task(async function testH3FallbackWithMultipleTransactions() {
     let internal = req.QueryInterface(Ci.nsIHttpChannelInternal);
     Assert.equal(internal.remotePort, h2Port);
   });
-
-  await trrServer.stop();
 });
 
 add_task(async function testTwoFastFallbackTimers() {
-  trrServer = new TRRServer();
-  await trrServer.start();
   Services.prefs.setBoolPref("network.dns.upgrade_with_https_rr", true);
   Services.prefs.setBoolPref("network.dns.use_https_rr_as_altsvc", true);
   Services.prefs.setBoolPref("network.dns.echconfig.enabled", false);
@@ -700,7 +683,7 @@ add_task(async function testTwoFastFallbackTimers() {
   Services.prefs.setIntPref("network.trr.mode", 3);
   Services.prefs.setCharPref(
     "network.trr.uri",
-    `https://foo.example.com:${trrServer.port}/dns-query`
+    `https://foo.example.com:${trrServer.port()}/dns-query`
   );
   Services.prefs.setBoolPref("network.http.http3.enable", true);
 
@@ -711,7 +694,7 @@ add_task(async function testTwoFastFallbackTimers() {
 
   Services.prefs.setCharPref(
     "network.http.http3.alt-svc-mapping-for-testing",
-    "foo.fallback.org;h3-29=:" + h3Port
+    "foo.fallback.org;h3=:" + h3Port
   );
 
   Services.prefs.setIntPref(
@@ -731,7 +714,7 @@ add_task(async function testTwoFastFallbackTimers() {
           priority: 1,
           name: "foo.fallback.org",
           values: [
-            { key: "alpn", value: "h3-29" },
+            { key: "alpn", value: "h3" },
             { key: "port", value: h3Port },
           ],
         },
@@ -781,13 +764,9 @@ add_task(async function testTwoFastFallbackTimers() {
   Services.prefs.setIntPref("network.http.http3.backup_timer_delay", 10);
 
   await createChannelAndStartTest();
-
-  await trrServer.stop();
 });
 
 add_task(async function testH3FastFallbackWithMultipleTransactions() {
-  trrServer = new TRRServer();
-  await trrServer.start();
   Services.prefs.setBoolPref("network.dns.upgrade_with_https_rr", true);
   Services.prefs.setBoolPref("network.dns.use_https_rr_as_altsvc", true);
   Services.prefs.setBoolPref("network.dns.echconfig.enabled", false);
@@ -795,7 +774,7 @@ add_task(async function testH3FastFallbackWithMultipleTransactions() {
   Services.prefs.setIntPref("network.trr.mode", 3);
   Services.prefs.setCharPref(
     "network.trr.uri",
-    `https://foo.example.com:${trrServer.port}/dns-query`
+    `https://foo.example.com:${trrServer.port()}/dns-query`
   );
   Services.prefs.setBoolPref("network.http.http3.enable", true);
 
@@ -808,7 +787,7 @@ add_task(async function testH3FastFallbackWithMultipleTransactions() {
 
   Services.prefs.setCharPref(
     "network.http.http3.alt-svc-mapping-for-testing",
-    "test.multiple_fallback_trans.org;h3-29=:" + h3Port
+    "test.multiple_fallback_trans.org;h3=:" + h3Port
   );
 
   await trrServer.registerDoHAnswers("test.multiple_fallback_trans.org", "A", {
@@ -842,13 +821,9 @@ add_task(async function testH3FastFallbackWithMultipleTransactions() {
     let internal = req.QueryInterface(Ci.nsIHttpChannelInternal);
     Assert.equal(internal.remotePort, h2Port);
   });
-
-  await trrServer.stop();
 });
 
 add_task(async function testFastfallbackToTheSameRecord() {
-  trrServer = new TRRServer();
-  await trrServer.start();
   Services.prefs.setBoolPref("network.dns.upgrade_with_https_rr", true);
   Services.prefs.setBoolPref("network.dns.use_https_rr_as_altsvc", true);
   Services.prefs.setBoolPref("network.dns.echconfig.enabled", true);
@@ -857,7 +832,7 @@ add_task(async function testFastfallbackToTheSameRecord() {
   Services.prefs.setIntPref("network.trr.mode", 3);
   Services.prefs.setCharPref(
     "network.trr.uri",
-    `https://foo.example.com:${trrServer.port}/dns-query`
+    `https://foo.example.com:${trrServer.port()}/dns-query`
   );
   Services.prefs.setBoolPref("network.http.http3.enable", true);
 
@@ -877,7 +852,7 @@ add_task(async function testFastfallbackToTheSameRecord() {
           priority: 1,
           name: "test.ech1.org",
           values: [
-            { key: "alpn", value: ["h3-29", "h2"] },
+            { key: "alpn", value: ["h3", "h2"] },
             { key: "port", value: h2Port },
             { key: "echconfig", value: "456..." },
           ],
@@ -903,6 +878,61 @@ add_task(async function testFastfallbackToTheSameRecord() {
   Assert.equal(req.protocolVersion, "h2");
   let internal = req.QueryInterface(Ci.nsIHttpChannelInternal);
   Assert.equal(internal.remotePort, h2Port);
+});
 
-  await trrServer.stop();
+// Similar to the previous test, but no ech.
+add_task(async function testFastfallbackToTheSameRecord1() {
+  Services.prefs.setBoolPref("network.dns.upgrade_with_https_rr", true);
+  Services.prefs.setBoolPref("network.dns.use_https_rr_as_altsvc", true);
+  Services.prefs.setBoolPref("network.dns.echconfig.enabled", true);
+  Services.prefs.setBoolPref("network.dns.http3_echconfig.enabled", true);
+
+  Services.prefs.setIntPref("network.trr.mode", 3);
+  Services.prefs.setCharPref(
+    "network.trr.uri",
+    `https://foo.example.com:${trrServer.port()}/dns-query`
+  );
+  Services.prefs.setBoolPref("network.http.http3.enable", true);
+
+  Services.prefs.setIntPref(
+    "network.dns.httpssvc.http3_fast_fallback_timeout",
+    1000
+  );
+
+  await trrServer.registerDoHAnswers("test.no_ech.org", "HTTPS", {
+    answers: [
+      {
+        name: "test.no_ech.org",
+        ttl: 55,
+        type: "HTTPS",
+        flush: false,
+        data: {
+          priority: 1,
+          name: "test.no_ech.org",
+          values: [
+            { key: "alpn", value: ["h3", "h2"] },
+            { key: "port", value: h2Port },
+          ],
+        },
+      },
+    ],
+  });
+
+  await trrServer.registerDoHAnswers("test.no_ech.org", "A", {
+    answers: [
+      {
+        name: "test.no_ech.org",
+        ttl: 55,
+        type: "A",
+        flush: false,
+        data: "127.0.0.1",
+      },
+    ],
+  });
+
+  let chan = makeChan(`https://test.no_ech.org/server-timing`);
+  let [req] = await channelOpenPromise(chan);
+  Assert.equal(req.protocolVersion, "h2");
+  let internal = req.QueryInterface(Ci.nsIHttpChannelInternal);
+  Assert.equal(internal.remotePort, h2Port);
 });
