@@ -4,10 +4,8 @@
 
 "use strict";
 
-ChromeUtils.import("resource://gre/modules/NetUtil.jsm");
-var { Services } = ChromeUtils.import("resource://gre/modules/Services.jsm");
-const { TestUtils } = ChromeUtils.import(
-  "resource://testing-common/TestUtils.jsm"
+const { TestUtils } = ChromeUtils.importESModule(
+  "resource://testing-common/TestUtils.sys.mjs"
 );
 
 let h2Port;
@@ -17,48 +15,69 @@ function inChildProcess() {
   return Services.appinfo.processType != Ci.nsIXULRuntime.PROCESS_TYPE_DEFAULT;
 }
 
-const dns = Cc["@mozilla.org/network/dns-service;1"].getService(
-  Ci.nsIDNSService
-);
+async function clearCache() {
+  if (inChildProcess()) {
+    do_send_remote_message("clearCache");
+    await do_await_remote_message("clearCache-done");
+  } else {
+    Services.dns.clearCache(true);
+  }
+}
+
+async function setTRRURI(uri) {
+  if (!inChildProcess()) {
+    Services.prefs.setCharPref("network.trr.uri", uri);
+    Services.prefs.setIntPref("network.trr.mode", 3);
+  } else {
+    do_send_remote_message("set-trr-uri", uri);
+    await do_await_remote_message("set-trr-uri-done");
+  }
+}
 
 add_setup(async function setup() {
+  trrServer = new TRRServer();
+  await trrServer.start();
+  h2Port = trrServer.port();
+  ok(h2Port);
+
+  registerCleanupFunction(async () => {
+    if (trrServer) {
+      await trrServer.stop();
+    }
+    if (inChildProcess()) {
+      do_send_remote_message("set-trr-uri", "test-done");
+      do_send_remote_message("clearCache", "test-done");
+    }
+  });
+
   if (inChildProcess()) {
     return;
   }
 
   trr_test_setup();
-  let env = Cc["@mozilla.org/process/environment;1"].getService(
-    Ci.nsIEnvironment
-  );
-  h2Port = env.get("MOZHTTP2_PORT");
-  Assert.notEqual(h2Port, null);
-  Assert.notEqual(h2Port, "");
 
   registerCleanupFunction(async () => {
     trr_clear_prefs();
     Services.prefs.clearUserPref("network.dns.port_prefixed_qname_https_rr");
-    await trrServer.stop();
   });
 
   if (mozinfo.socketprocess_networking) {
+    Services.dns; // Needed to trigger socket process.
     await TestUtils.waitForCondition(() => Services.io.socketProcessLaunched);
   }
-
-  Services.prefs.setIntPref("network.trr.mode", 3);
 });
 
 add_task(async function testHTTPSSVC() {
   // use the h2 server as DOH provider
-  if (!inChildProcess()) {
-    Services.prefs.setCharPref(
-      "network.trr.uri",
-      "https://foo.example.com:" + h2Port + "/httpssvc"
-    );
-  }
+  await setTRRURI("https://foo.example.com:" + h2Port + "/doh?httpssvc=1");
 
   let { inRecord } = await new TRRDNSListener("test.httpssvc.com", {
-    type: dns.RESOLVE_TYPE_HTTPSSVC,
+    type: Ci.nsIDNSService.RESOLVE_TYPE_HTTPSSVC,
   });
+  Assert.ok(
+    inRecord.QueryInterface(Ci.nsIDNSHTTPSSVCRecord).IsTRR(),
+    "resolved by TRR"
+  );
   let answer = inRecord.QueryInterface(Ci.nsIDNSHTTPSSVCRecord).records;
   Assert.equal(answer[0].priority, 1);
   Assert.equal(answer[0].name, "h3pool");
@@ -147,23 +166,49 @@ add_task(async function testHTTPSSVC() {
 });
 
 add_task(async function test_aliasform() {
-  trrServer = new TRRServer();
-  await trrServer.start();
-  dump(`port = ${trrServer.port}\n`);
+  await setTRRURI(`https://foo.example.com:${trrServer.port()}/dns-query`);
+  await clearCache();
 
-  if (inChildProcess()) {
-    do_send_remote_message("mode3-port", trrServer.port);
-    await do_await_remote_message("mode3-port-done");
-  } else {
-    Services.prefs.setIntPref("network.trr.mode", 3);
-    Services.prefs.setCharPref(
-      "network.trr.uri",
-      `https://foo.example.com:${trrServer.port}/dns-query`
+  // Make sure that HTTPS AliasForm is only treated as a CNAME for HTTPS requests
+  await trrServer.registerDoHAnswers("test1.com", "A", {
+    answers: [
+      {
+        name: "test1.com",
+        ttl: 55,
+        type: "HTTPS",
+        flush: false,
+        data: {
+          priority: 0,
+          name: "something1.com",
+          values: [],
+        },
+      },
+    ],
+  });
+  await trrServer.registerDoHAnswers("something1.com", "A", {
+    answers: [
+      {
+        name: "something1.com",
+        ttl: 55,
+        type: "A",
+        flush: false,
+        data: "1.2.3.4",
+      },
+    ],
+  });
+
+  {
+    let { inStatus } = await new TRRDNSListener("test1.com", {
+      expectedSuccess: false,
+    });
+    Assert.ok(
+      !Components.isSuccessCode(inStatus),
+      `${inStatus} should be an error code`
     );
   }
 
   // Test that HTTPS priority = 0 (AliasForm) behaves like a CNAME
-  await trrServer.registerDoHAnswers("test.com", "A", {
+  await trrServer.registerDoHAnswers("test.com", "HTTPS", {
     answers: [
       {
         name: "test.com",
@@ -178,22 +223,35 @@ add_task(async function test_aliasform() {
       },
     ],
   });
-  await trrServer.registerDoHAnswers("something.com", "A", {
+  await trrServer.registerDoHAnswers("something.com", "HTTPS", {
     answers: [
       {
         name: "something.com",
         ttl: 55,
-        type: "A",
+        type: "HTTPS",
         flush: false,
-        data: "1.2.3.4",
+        data: {
+          priority: 1,
+          name: "h3pool",
+          values: [{ key: "alpn", value: ["h2", "h3"] }],
+        },
       },
     ],
   });
 
-  await new TRRDNSListener("test.com", { expectedAnswer: "1.2.3.4" });
+  {
+    let { inStatus, inRecord } = await new TRRDNSListener("test.com", {
+      type: Ci.nsIDNSService.RESOLVE_TYPE_HTTPSSVC,
+      expectedSuccess: false,
+    });
+    Assert.ok(Components.isSuccessCode(inStatus), `${inStatus} should succeed`);
+    let answer = inRecord.QueryInterface(Ci.nsIDNSHTTPSSVCRecord).records;
+    Assert.equal(answer[0].priority, 1);
+    Assert.equal(answer[0].name, "h3pool");
+  }
 
   // Test a chain of HTTPSSVC AliasForm and CNAMEs
-  await trrServer.registerDoHAnswers("x.com", "A", {
+  await trrServer.registerDoHAnswers("x.com", "HTTPS", {
     answers: [
       {
         name: "x.com",
@@ -208,7 +266,7 @@ add_task(async function test_aliasform() {
       },
     ],
   });
-  await trrServer.registerDoHAnswers("y.com", "A", {
+  await trrServer.registerDoHAnswers("y.com", "HTTPS", {
     answers: [
       {
         name: "y.com",
@@ -220,7 +278,7 @@ add_task(async function test_aliasform() {
       },
     ],
   });
-  await trrServer.registerDoHAnswers("z.com", "A", {
+  await trrServer.registerDoHAnswers("z.com", "HTTPS", {
     answers: [
       {
         name: "z.com",
@@ -235,19 +293,30 @@ add_task(async function test_aliasform() {
       },
     ],
   });
-  await trrServer.registerDoHAnswers("target.com", "A", {
+  await trrServer.registerDoHAnswers("target.com", "HTTPS", {
     answers: [
       {
         name: "target.com",
         ttl: 55,
-        type: "A",
+        type: "HTTPS",
         flush: false,
-        data: "4.3.2.1",
+        data: {
+          priority: 1,
+          name: "h3pool",
+          values: [{ key: "alpn", value: ["h2", "h3"] }],
+        },
       },
     ],
   });
 
-  await new TRRDNSListener("x.com", { expectedAnswer: "4.3.2.1" });
+  let { inStatus, inRecord } = await new TRRDNSListener("x.com", {
+    type: Ci.nsIDNSService.RESOLVE_TYPE_HTTPSSVC,
+    expectedSuccess: false,
+  });
+  Assert.ok(Components.isSuccessCode(inStatus), `${inStatus} should succeed`);
+  let answer = inRecord.QueryInterface(Ci.nsIDNSHTTPSSVCRecord).records;
+  Assert.equal(answer[0].priority, 1);
+  Assert.equal(answer[0].name, "h3pool");
 
   // We get a ServiceForm instead of a A answer, CNAME or AliasForm
   await trrServer.registerDoHAnswers("no-ip-host.com", "A", {
@@ -273,16 +342,16 @@ add_task(async function test_aliasform() {
     ],
   });
 
-  let { inStatus } = await new TRRDNSListener("no-ip-host.com", {
+  ({ inStatus } = await new TRRDNSListener("no-ip-host.com", {
     expectedSuccess: false,
-  });
+  }));
   Assert.ok(
     !Components.isSuccessCode(inStatus),
     `${inStatus} should be an error code`
   );
 
   // Test CNAME/AliasForm loop
-  await trrServer.registerDoHAnswers("loop.com", "A", {
+  await trrServer.registerDoHAnswers("loop.com", "HTTPS", {
     answers: [
       {
         name: "loop.com",
@@ -294,7 +363,7 @@ add_task(async function test_aliasform() {
       },
     ],
   });
-  await trrServer.registerDoHAnswers("loop2.com", "A", {
+  await trrServer.registerDoHAnswers("loop2.com", "HTTPS", {
     answers: [
       {
         name: "loop2.com",
@@ -310,13 +379,21 @@ add_task(async function test_aliasform() {
     ],
   });
 
+  // Make sure these are the first requests
+  Assert.equal(await trrServer.requestCount("loop.com", "HTTPS"), 0);
+  Assert.equal(await trrServer.requestCount("loop2.com", "HTTPS"), 0);
+
   ({ inStatus } = await new TRRDNSListener("loop.com", {
+    type: Ci.nsIDNSService.RESOLVE_TYPE_HTTPSSVC,
     expectedSuccess: false,
   }));
   Assert.ok(
     !Components.isSuccessCode(inStatus),
     `${inStatus} should be an error code`
   );
+  // Make sure the error was actually triggered by a loop.
+  Assert.greater(await trrServer.requestCount("loop.com", "HTTPS"), 2);
+  Assert.greater(await trrServer.requestCount("loop2.com", "HTTPS"), 2);
 
   // Alias form for .
   await trrServer.registerDoHAnswers("empty.com", "A", {
@@ -379,7 +456,7 @@ add_task(async function test_aliasform() {
   });
 
   let { inStatus: inStatus2 } = await new TRRDNSListener("multi.com", {
-    type: dns.RESOLVE_TYPE_HTTPSSVC,
+    type: Ci.nsIDNSService.RESOLVE_TYPE_HTTPSSVC,
     expectedSuccess: false,
   });
   Assert.ok(
@@ -412,7 +489,7 @@ add_task(async function test_aliasform() {
   });
 
   ({ inStatus: inStatus2 } = await new TRRDNSListener("order.com", {
-    type: dns.RESOLVE_TYPE_HTTPSSVC,
+    type: Ci.nsIDNSService.RESOLVE_TYPE_HTTPSSVC,
     expectedSuccess: false,
   }));
   Assert.ok(
@@ -441,7 +518,7 @@ add_task(async function test_aliasform() {
   });
 
   ({ inStatus: inStatus2 } = await new TRRDNSListener("duplicate.com", {
-    type: dns.RESOLVE_TYPE_HTTPSSVC,
+    type: Ci.nsIDNSService.RESOLVE_TYPE_HTTPSSVC,
     expectedSuccess: false,
   }));
   Assert.ok(
@@ -471,7 +548,7 @@ add_task(async function test_aliasform() {
   });
 
   ({ inStatus: inStatus2 } = await new TRRDNSListener("mandatory.com", {
-    type: dns.RESOLVE_TYPE_HTTPSSVC,
+    type: Ci.nsIDNSService.RESOLVE_TYPE_HTTPSSVC,
     expectedSuccess: false,
   }));
   Assert.ok(!Components.isSuccessCode(inStatus2), `${inStatus2} should fail`);
@@ -512,7 +589,7 @@ add_task(async function test_aliasform() {
   });
 
   ({ inStatus: inStatus2 } = await new TRRDNSListener("mandatory2.com", {
-    type: dns.RESOLVE_TYPE_HTTPSSVC,
+    type: Ci.nsIDNSService.RESOLVE_TYPE_HTTPSSVC,
   }));
 
   Assert.ok(Components.isSuccessCode(inStatus2), `${inStatus2} should succeed`);
@@ -535,7 +612,7 @@ add_task(async function test_aliasform() {
   });
 
   ({ inStatus: inStatus2 } = await new TRRDNSListener("no-alias.com", {
-    type: dns.RESOLVE_TYPE_HTTPSSVC,
+    type: Ci.nsIDNSService.RESOLVE_TYPE_HTTPSSVC,
     expectedSuccess: false,
   }));
 
@@ -558,19 +635,19 @@ add_task(async function test_aliasform() {
     ],
   });
 
-  let inRecord;
   ({ inRecord, inStatus: inStatus2 } = await new TRRDNSListener("service.com", {
-    type: dns.RESOLVE_TYPE_HTTPSSVC,
+    type: Ci.nsIDNSService.RESOLVE_TYPE_HTTPSSVC,
   }));
   Assert.ok(Components.isSuccessCode(inStatus2), `${inStatus2} should work`);
-  let answer = inRecord.QueryInterface(Ci.nsIDNSHTTPSSVCRecord).records;
+  answer = inRecord.QueryInterface(Ci.nsIDNSHTTPSSVCRecord).records;
   Assert.equal(answer[0].priority, 1);
   Assert.equal(answer[0].name, "service.com");
 });
 
 add_task(async function testNegativeResponse() {
+  await setTRRURI(`https://foo.example.com:${trrServer.port()}/dns-query`);
   let { inStatus } = await new TRRDNSListener("negative_test.com", {
-    type: dns.RESOLVE_TYPE_HTTPSSVC,
+    type: Ci.nsIDNSService.RESOLVE_TYPE_HTTPSSVC,
     expectedSuccess: false,
   });
   Assert.ok(
@@ -596,7 +673,7 @@ add_task(async function testNegativeResponse() {
 
   // Should still be failed because a negative response is from DNS cache.
   ({ inStatus } = await new TRRDNSListener("negative_test.com", {
-    type: dns.RESOLVE_TYPE_HTTPSSVC,
+    type: Ci.nsIDNSService.RESOLVE_TYPE_HTTPSSVC,
     expectedSuccess: false,
   }));
   Assert.ok(
@@ -604,16 +681,11 @@ add_task(async function testNegativeResponse() {
     `${inStatus} should be an error code`
   );
 
-  if (inChildProcess()) {
-    do_send_remote_message("clearCache");
-    await do_await_remote_message("clearCache-done");
-  } else {
-    dns.clearCache(true);
-  }
+  await clearCache();
 
   let inRecord;
   ({ inRecord, inStatus } = await new TRRDNSListener("negative_test.com", {
-    type: dns.RESOLVE_TYPE_HTTPSSVC,
+    type: Ci.nsIDNSService.RESOLVE_TYPE_HTTPSSVC,
   }));
   Assert.ok(Components.isSuccessCode(inStatus), `${inStatus} should work`);
   let answer = inRecord.QueryInterface(Ci.nsIDNSHTTPSSVCRecord).records;
@@ -655,7 +727,7 @@ add_task(async function testPortPrefixedName() {
   let { inRecord, inStatus } = await new TRRDNSListener(
     "port_prefix.test.com",
     {
-      type: dns.RESOLVE_TYPE_HTTPSSVC,
+      type: Ci.nsIDNSService.RESOLVE_TYPE_HTTPSSVC,
       port: 4433,
     }
   );
@@ -663,5 +735,172 @@ add_task(async function testPortPrefixedName() {
   let answer = inRecord.QueryInterface(Ci.nsIDNSHTTPSSVCRecord).records;
   Assert.equal(answer[0].priority, 1);
   Assert.equal(answer[0].name, "port_prefix.test1.com");
-  await trrServer.stop();
+});
+
+async function doTestFlattenRecordsWithECH(host, targetName, alpn, expected) {
+  await clearCache();
+
+  await trrServer.registerDoHAnswers(host, "HTTPS", {
+    answers: [
+      {
+        name: host,
+        ttl: 55,
+        type: "HTTPS",
+        flush: false,
+        data: {
+          priority: 1,
+          name: targetName,
+          values: [
+            ...(alpn ? [{ key: "alpn", value: alpn }] : []),
+            { key: "echconfig", value: "456..." },
+          ],
+        },
+      },
+    ],
+  });
+
+  let { inRecord } = await new TRRDNSListener(host, {
+    type: Ci.nsIDNSService.RESOLVE_TYPE_HTTPSSVC,
+  });
+
+  let notused = {};
+  let answer = inRecord
+    .QueryInterface(Ci.nsIDNSHTTPSSVCRecord)
+    .GetAllRecordsWithEchConfig(false, false, "", notused, notused);
+
+  Assert.equal(answer.length, expected.length);
+  for (let i = 0; i < answer.length; i++) {
+    Assert.equal(answer[i].priority, expected[i].priority);
+    Assert.equal(answer[i].name, expected[i].name);
+    if (expected[i].selectedAlpn !== null) {
+      Assert.equal(answer[i].selectedAlpn, expected[i].selectedAlpn);
+    }
+  }
+}
+
+add_task(async function testFlattenRecordsWithECH() {
+  // Test when host name and targetName are the same.
+  await doTestFlattenRecordsWithECH(
+    "test.target.com",
+    "test.target.com",
+    ["h3"],
+    [
+      { priority: 1, name: "test.target.com", selectedAlpn: "h3" },
+      { priority: 1, name: "test.target.com", selectedAlpn: "" },
+    ]
+  );
+  await doTestFlattenRecordsWithECH(
+    "test.target.com",
+    ".",
+    ["h3"],
+    [
+      { priority: 1, name: "test.target.com", selectedAlpn: "h3" },
+      { priority: 1, name: "test.target.com", selectedAlpn: "" },
+    ]
+  );
+
+  // Test when host name and targetName are not the same.
+  // We add
+  await doTestFlattenRecordsWithECH(
+    "test.target.com",
+    "test.target_1.com",
+    ["h3"],
+    [
+      { priority: 1, name: "test.target_1.com", selectedAlpn: "h3" },
+      { priority: 1, name: "test.target_1.com", selectedAlpn: "" },
+    ]
+  );
+
+  // Test when alpn is empty.
+  await doTestFlattenRecordsWithECH("test.target.com", ".", null, [
+    { priority: 1, name: "test.target.com", selectedAlpn: null },
+  ]);
+  await doTestFlattenRecordsWithECH(
+    "test.target.com",
+    "test.target_1.com",
+    null,
+    [{ priority: 1, name: "test.target_1.com", selectedAlpn: null }]
+  );
+});
+
+async function doTestFlattenRecordsWithoutECH(
+  host,
+  targetName,
+  alpn,
+  expected
+) {
+  await clearCache();
+
+  await setTRRURI(`https://foo.example.com:${trrServer.port()}/dns-query`);
+  await trrServer.registerDoHAnswers(host, "HTTPS", {
+    answers: [
+      {
+        name: host,
+        ttl: 55,
+        type: "HTTPS",
+        flush: false,
+        data: {
+          priority: 1,
+          name: targetName,
+          values: [...(alpn ? [{ key: "alpn", value: alpn }] : [])],
+        },
+      },
+    ],
+  });
+
+  let { inRecord } = await new TRRDNSListener(host, {
+    type: Ci.nsIDNSService.RESOLVE_TYPE_HTTPSSVC,
+  });
+
+  let notused = {};
+  let answer = inRecord
+    .QueryInterface(Ci.nsIDNSHTTPSSVCRecord)
+    .GetAllRecords(false, false, "", notused, notused);
+
+  Assert.equal(answer.length, expected.length);
+  for (let i = 0; i < answer.length; i++) {
+    Assert.equal(answer[i].priority, expected[i].priority);
+    Assert.equal(answer[i].name, expected[i].name);
+    if (expected[i].selectedAlpn !== null) {
+      Assert.equal(answer[i].selectedAlpn, expected[i].selectedAlpn);
+    }
+  }
+}
+
+add_task(async function testFlattenRecordsWithoutECH() {
+  // Test when host name and targetName are the same.
+  await doTestFlattenRecordsWithoutECH(
+    "test.target_noech.com",
+    "test.target_noech.com",
+    ["h3"],
+    [{ priority: 1, name: "test.target_noech.com", selectedAlpn: "h3" }]
+  );
+  await doTestFlattenRecordsWithoutECH(
+    "test.target_noech.com",
+    ".",
+    ["h3"],
+    [{ priority: 1, name: "test.target_noech.com", selectedAlpn: "h3" }]
+  );
+
+  // Test when host name and targetName are not the same.
+  await doTestFlattenRecordsWithoutECH(
+    "test.target_noech.com",
+    "test.target_noech_1.com",
+    ["h3"],
+    [
+      { priority: 1, name: "test.target_noech_1.com", selectedAlpn: "h3" },
+      { priority: 1, name: "test.target_noech_1.com", selectedAlpn: "" },
+    ]
+  );
+
+  // Test when alpn is empty.
+  await doTestFlattenRecordsWithoutECH("test.target_noech.com", ".", null, [
+    { priority: 1, name: "test.target_noech.com", selectedAlpn: null },
+  ]);
+  await doTestFlattenRecordsWithoutECH(
+    "test.target_noech.com",
+    "test.target_noech_1.com",
+    null,
+    [{ priority: 1, name: "test.target_noech_1.com", selectedAlpn: null }]
+  );
 });
