@@ -1,4 +1,6 @@
-/* globals dnsResolve */
+// These are globlas defined for proxy servers, in ProxyAutoConfig.cpp. See
+// PACGlobalFunctions
+/* globals dnsResolve, alert */
 
 /* This test checks that using a PAC script still works when TRR is on.
    Steps:
@@ -9,12 +11,11 @@
    We run these steps for TRR mode 2 and 3, and with fetchOffMainThread = true/false
 */
 
-const { HttpServer } = ChromeUtils.import("resource://testing-common/httpd.js");
-const { MockRegistrar } = ChromeUtils.import(
-  "resource://testing-common/MockRegistrar.jsm"
+const { HttpServer } = ChromeUtils.importESModule(
+  "resource://testing-common/httpd.sys.mjs"
 );
-const dns = Cc["@mozilla.org/network/dns-service;1"].getService(
-  Ci.nsIDNSService
+const { MockRegistrar } = ChromeUtils.importESModule(
+  "resource://testing-common/MockRegistrar.sys.mjs"
 );
 
 registerCleanupFunction(async () => {
@@ -29,8 +30,7 @@ function FindProxyForURL(url, host) {
   return "DIRECT";
 }
 
-const CID = Components.ID("{5645d2c1-d6d8-4091-b117-fe7ee4027db7}");
-XPCOMUtils.defineLazyGetter(this, "systemSettings", function() {
+ChromeUtils.defineLazyGetter(this, "systemSettings", function () {
   return {
     QueryInterface: ChromeUtils.generateQI(["nsISystemProxySettings"]),
 
@@ -38,7 +38,7 @@ XPCOMUtils.defineLazyGetter(this, "systemSettings", function() {
     PACURI: `data:application/x-ns-proxy-autoconfig;charset=utf-8,${encodeURIComponent(
       FindProxyForURL.toString()
     )}`,
-    getProxyForURI(aURI) {
+    getProxyForURI() {
       throw Components.Exception("", Cr.NS_ERROR_NOT_IMPLEMENTED);
     },
   };
@@ -97,27 +97,25 @@ async function do_test_pac_dnsResolve() {
   await new Promise(resolve => chan.asyncOpen(new ChannelListener(resolve)));
   await consolePromise;
 
-  let env = Cc["@mozilla.org/process/environment;1"].getService(
-    Ci.nsIEnvironment
-  );
-  let h2Port = env.get("MOZHTTP2_PORT");
-  Assert.notEqual(h2Port, null);
-  Assert.notEqual(h2Port, "");
+  let trrServer = new TRRServer();
+  await trrServer.start();
+
+  registerCleanupFunction(async () => {
+    if (trrServer) {
+      await trrServer.stop();
+    }
+  });
 
   override.addIPOverride("foo.example.com", "127.0.0.1");
   Services.prefs.setCharPref(
     "network.trr.uri",
-    `https://foo.example.com:${h2Port}/doh?responseIP=127.0.0.1`
+    `https://foo.example.com:${trrServer.port()}/doh?responseIP=127.0.0.1`
   );
 
   trr_test_setup();
 
-  async function test_with(DOMAIN, trrMode, fetchOffMainThread) {
+  async function test_with(DOMAIN, trrMode) {
     Services.prefs.setIntPref("network.trr.mode", trrMode); // TRR first
-    Services.prefs.setBoolPref(
-      "network.trr.fetch_off_main_thread",
-      fetchOffMainThread
-    );
     override.addIPOverride(DOMAIN, "127.0.0.1");
 
     chan = NetUtil.newChannel({
@@ -129,11 +127,13 @@ async function do_test_pac_dnsResolve() {
     await override.clearHostOverride(DOMAIN);
   }
 
-  await test_with("test1.com", 2, true);
-  await test_with("test2.com", 3, true);
-  await test_with("test3.com", 2, false);
-  await test_with("test4.com", 3, false);
+  await test_with("test1.com", 2);
+  await test_with("test2.com", 3);
   await httpserv.stop();
+  if (trrServer) {
+    await trrServer.stop();
+    trrServer = null;
+  }
 }
 
 add_task(async function test_pac_dnsResolve() {
