@@ -13,7 +13,9 @@ const { TestUtils } = ChromeUtils.importESModule(
   "resource://testing-common/TestUtils.sys.mjs"
 );
 
-const { HttpServer } = ChromeUtils.import("resource://testing-common/httpd.js");
+const { HttpServer } = ChromeUtils.importESModule(
+  "resource://testing-common/httpd.sys.mjs"
+);
 
 const TRR_Domain = "foo.example.com";
 
@@ -467,6 +469,8 @@ async function test_mode_1_and_4() {
 
 async function test_CNAME() {
   info("Checking that we follow a CNAME correctly");
+  // cnameHandler for dns-cname in head_trr doesn't currently suppport GET
+  Services.prefs.setBoolPref("network.trr.useGET", false);
   Services.dns.clearCache(true);
   // The dns-cname path alternates between sending us a CNAME pointing to
   // another domain, and an A record. If we follow the cname correctly, doing
@@ -502,6 +506,7 @@ async function test_CNAME() {
   setModeAndURI(3, "dns-cname-a");
 
   await new TRRDNSListener("cname-a.example.com", "9.8.7.6");
+  Services.prefs.clearUserPref("network.trr.useGET");
 }
 
 async function test_name_mismatch() {
@@ -1025,6 +1030,8 @@ async function test_ipv4_trr_fallback() {
 async function test_no_retry_without_doh() {
   info("Bug 1648147 - if the TRR returns 0.0.0.0 we should not retry with DNS");
   Services.prefs.setBoolPref("network.trr.fallback-on-zero-response", false);
+  Services.prefs.setBoolPref("network.socket.ip_addr_any.disabled", false);
+  Services.prefs.setBoolPref("network.trr.allow-rfc1918", true);
 
   async function test(url, ip) {
     setModeAndURI(2, `doh?responseIP=${ip}`);
@@ -1041,8 +1048,8 @@ async function test_no_retry_without_doh() {
       getInterface(iid) {
         return this.QueryInterface(iid);
       },
-      onProgress(request, progress, progressMax) {},
-      onStatus(request, status, statusArg) {
+      onProgress() {},
+      onStatus(request, status) {
         this.statusCount[status] = 1 + (this.statusCount[status] || 0);
       },
     };
@@ -1071,163 +1078,7 @@ async function test_no_retry_without_doh() {
     await test(`http://unknown.ipv4.stuff:666/path`, "0.0.0.0");
     await test(`http://unknown.ipv6.stuff:666/path`, "::");
   }
-}
 
-async function test_connection_reuse_and_cycling() {
-  Services.dns.clearCache(true);
-  Services.prefs.setIntPref("network.trr.request_timeout_ms", 500);
-  Services.prefs.setIntPref(
-    "network.trr.strict_fallback_request_timeout_ms",
-    500
-  );
-  Services.prefs.setIntPref("network.trr.request_timeout_mode_trronly_ms", 500);
-
-  setModeAndURI(2, `doh?responseIP=9.8.7.6`);
-  Services.prefs.setBoolPref("network.trr.strict_native_fallback", true);
-  Services.prefs.setCharPref("network.trr.confirmationNS", "example.com");
-  await TestUtils.waitForCondition(
-    // 2 => CONFIRM_OK
-    () => Services.dns.currentTrrConfirmationState == 2,
-    `Timed out waiting for confirmation success. Currently ${Services.dns.currentTrrConfirmationState}`,
-    1,
-    5000
-  );
-
-  // Setting conncycle=true in the URI. Server will start logging reqs.
-  // We will do a specific sequence of lookups, then fetch the log from
-  // the server and check that it matches what we'd expect.
-  setModeAndURI(2, `doh?responseIP=9.8.7.6&conncycle=true`);
-  await TestUtils.waitForCondition(
-    // 2 => CONFIRM_OK
-    () => Services.dns.currentTrrConfirmationState == 2,
-    `Timed out waiting for confirmation success. Currently ${Services.dns.currentTrrConfirmationState}`,
-    1,
-    5000
-  );
-  // Confirmation upon uri-change will have created one req.
-
-  // Two reqs for each bar1 and bar2 - A + AAAA.
-  await new TRRDNSListener("bar1.example.org.", "9.8.7.6");
-  await new TRRDNSListener("bar2.example.org.", "9.8.7.6");
-  // Total so far: (1) + 2 + 2 = 5
-
-  // Two reqs that fail, one Confirmation req, two retried reqs that succeed.
-  await new TRRDNSListener("newconn.example.org.", "9.8.7.6");
-  await TestUtils.waitForCondition(
-    // 2 => CONFIRM_OK
-    () => Services.dns.currentTrrConfirmationState == 2,
-    `Timed out waiting for confirmation success. Currently ${Services.dns.currentTrrConfirmationState}`,
-    1,
-    5000
-  );
-  // Total so far: (5) + 2 + 1 + 2 = 10
-
-  // Two reqs for each bar3 and bar4 .
-  await new TRRDNSListener("bar3.example.org.", "9.8.7.6");
-  await new TRRDNSListener("bar4.example.org.", "9.8.7.6");
-  // Total so far: (10) + 2 + 2 = 14.
-
-  // Two reqs that fail, one Confirmation req, two retried reqs that succeed.
-  await new TRRDNSListener("newconn2.example.org.", "9.8.7.6");
-  await TestUtils.waitForCondition(
-    // 2 => CONFIRM_OK
-    () => Services.dns.currentTrrConfirmationState == 2,
-    `Timed out waiting for confirmation success. Currently ${Services.dns.currentTrrConfirmationState}`,
-    1,
-    5000
-  );
-  // Total so far: (14) + 2 + 1 + 2 = 19
-
-  // Two reqs for each bar5 and bar6 .
-  await new TRRDNSListener("bar5.example.org.", "9.8.7.6");
-  await new TRRDNSListener("bar6.example.org.", "9.8.7.6");
-  // Total so far: (19) + 2 + 2 = 23
-
-  let chan = makeChan(
-    `https://foo.example.com:${h2Port}/get-doh-req-port-log`,
-    Ci.nsIRequest.TRR_DISABLED_MODE
-  );
-  let dohReqPortLog = await new Promise(resolve =>
-    chan.asyncOpen(
-      new ChannelListener((stuff, buffer) => {
-        resolve(JSON.parse(buffer));
-      })
-    )
-  );
-
-  // Since the actual ports seen will vary at runtime, we use placeholders
-  // instead in our expected output definition. For example, if two entries
-  // both have "port1", it means they both should have the same port in the
-  // server's log.
-  // For reqs that fail and trigger a Confirmation + retry, the retried reqs
-  // might not re-use the new connection created for Confirmation due to a
-  // race, so we have an extra alternate expected port for them. This lets
-  // us test that they use *a* new port even if it's not *the* new port.
-  // Subsequent lookups are not affected, they will use the same conn as
-  // the Confirmation req.
-  let expectedLogTemplate = [
-    ["example.com", "port1"],
-    ["bar1.example.org", "port1"],
-    ["bar1.example.org", "port1"],
-    ["bar2.example.org", "port1"],
-    ["bar2.example.org", "port1"],
-    ["newconn.example.org", "port1"],
-    ["newconn.example.org", "port1"],
-    ["example.com", "port2"],
-    ["newconn.example.org", "port2"],
-    ["newconn.example.org", "port2"],
-    ["bar3.example.org", "port2"],
-    ["bar3.example.org", "port2"],
-    ["bar4.example.org", "port2"],
-    ["bar4.example.org", "port2"],
-    ["newconn2.example.org", "port2"],
-    ["newconn2.example.org", "port2"],
-    ["example.com", "port3"],
-    ["newconn2.example.org", "port3"],
-    ["newconn2.example.org", "port3"],
-    ["bar5.example.org", "port3"],
-    ["bar5.example.org", "port3"],
-    ["bar6.example.org", "port3"],
-    ["bar6.example.org", "port3"],
-  ];
-
-  if (expectedLogTemplate.length != dohReqPortLog.length) {
-    // This shouldn't happen, and if it does, we'll fail the assertion
-    // below. But first dump the whole server-side log to help with
-    // debugging should we see a failure. Most likely cause would be
-    // that another consumer of TRR happened to make a request while
-    // the test was running and polluted the log.
-    info(dohReqPortLog);
-  }
-
-  equal(
-    expectedLogTemplate.length,
-    dohReqPortLog.length,
-    "Correct number of req log entries"
-  );
-
-  let seenPorts = new Set();
-  // This is essentially a symbol table - as we iterate through the log
-  // we will assign the actual seen port numbers to the placeholders.
-  let seenPortsByExpectedPort = new Map();
-
-  for (let i = 0; i < expectedLogTemplate.length; i++) {
-    let expectedName = expectedLogTemplate[i][0];
-    let expectedPort = expectedLogTemplate[i][1];
-    let seenName = dohReqPortLog[i][0];
-    let seenPort = dohReqPortLog[i][1];
-    info(`Checking log entry. Name: ${seenName}, Port: ${seenPort}`);
-    equal(expectedName, seenName, "Name matches for entry " + i);
-    if (!seenPortsByExpectedPort.has(expectedPort)) {
-      ok(!seenPorts.has(seenPort), "Port should not have been previously used");
-      seenPorts.add(seenPort);
-      seenPortsByExpectedPort.set(expectedPort, seenPort);
-    } else {
-      equal(
-        seenPort,
-        seenPortsByExpectedPort.get(expectedPort),
-        "Connection was reused as expected"
-      );
-    }
-  }
+  Services.prefs.clearUserPref("network.trr.allow-rfc1918");
+  Services.prefs.clearUserPref("network.socket.ip_addr_any.disabled");
 }
